@@ -59,6 +59,8 @@ import (
 	featuregatetesting "k8s.io/component-base/featuregate/testing"
 	logsapi "k8s.io/component-base/logs/api/v1"
 	utilversion "k8s.io/component-base/version"
+	zpagesfeatures "k8s.io/component-base/zpages/features"
+	"k8s.io/component-base/zpages/flagz"
 	"k8s.io/klog/v2"
 	"k8s.io/kube-aggregator/pkg/apiserver"
 	"k8s.io/kubernetes/pkg/features"
@@ -213,7 +215,8 @@ func StartTestServer(t ktesting.TB, instanceOptions *TestServerInstanceOptions,
 		s.GenericServerRunOptions.RequestTimeout = instanceOptions.RequestTimeout
 	}
 
-	for _, f := range s.Flags().FlagSets {
+	namedFlagSets := s.Flags()
+	for _, f := range namedFlagSets.FlagSets {
 		fs.AddFlagSet(f)
 	}
 
@@ -356,6 +359,9 @@ func StartTestServer(t ktesting.TB, instanceOptions *TestServerInstanceOptions,
 	if err := fs.Parse(customFlags); err != nil {
 		return result, err
 	}
+	if utilfeature.DefaultFeatureGate.Enabled(zpagesfeatures.ComponentFlagz) {
+		s.Flagz = flagz.NamedFlagSetsReader{FlagSets: namedFlagSets}
+	}
 
 	// the RequestHeader options pointer gets replaced in the case of EnableCertAuth override
 	// and so flags are connected to a struct that no longer appears in the ServerOptions struct
