@@ -279,7 +279,7 @@ func TestConfigBaseMarshal(t *testing.T) {
 		`, kubeadmapiv1.SchemeGroupVersion.String())))
 
 		if expected != got {
-			t.Fatalf("Missmatch between expected and got:\nExpected:\n%s\n---\nGot:\n%s", expected, got)
+			t.Fatalf("Mismatch between expected and got:\nExpected:\n%s\n---\nGot:\n%s", expected, got)
 		}
 	})
 }
@@ -332,7 +332,7 @@ func TestGeneratedConfigFromCluster(t *testing.T) {
 				hash: testYAMLHash,
 			},
 			{
-				name:         "Missmatching hash means user supplied config",
+				name:         "Mismatching hash means user supplied config",
 				hash:         mismatchHash,
 				userSupplied: true,
 			},
