@@ -36,26 +36,14 @@ if [[ -n "${direct_sets}" ]]; then
   echo "${direct_sets}" >&2
   echo >&2
   echo "Use this invocation instead:" >&2
-  echo "  defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.<FeatureName>, <value>)()" >&2
-  echo >&2
-  rc=1
-fi
-
-# find test files calling SetFeatureGateDuringTest and not calling the result
-missing_defers=$(git grep "\\.SetFeatureGateDuringTest" -- '*_test.go' | grep -E -v "defer .*\\)\\(\\)$") || true
-if [[ -n "${missing_defers}" ]]; then
-  echo "Invalid invocations of featuregatetesting.SetFeatureGateDuringTest():" >&2
-  echo "${missing_defers}" >&2
-  echo >&2
-  echo "Always make a deferred call to the returned function to ensure the feature gate is reset:" >&2
-  echo "  defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.<FeatureName>, <value>)()" >&2
+  echo "  featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.<FeatureName>, <value>)" >&2
   echo >&2
   rc=1
 fi
 
 
 # ensure all generic features are added in alphabetic order
-lines=$(git grep 'genericfeatures[.].*:' -- pkg/features/kube_features.go)
+lines=$(git grep 'genericfeatures[.].*:' -- pkg/features/versioned_kube_features.go)
 sorted_lines=$(echo "$lines" | sort -f)
 if [[ "$lines" != "$sorted_lines" ]]; then
   echo "Generic features in pkg/features/kube_features.go not sorted" >&2
