@@ -22,7 +22,7 @@ import (
 	"k8s.io/apimachinery/pkg/util/intstr"
 	utilfeature "k8s.io/apiserver/pkg/util/feature"
 	"k8s.io/kubernetes/pkg/features"
-	"k8s.io/utils/pointer"
+	"k8s.io/utils/ptr"
 )
 
 func addDefaultingFuncs(scheme *runtime.Scheme) error {
@@ -41,13 +41,11 @@ func SetDefaults_DaemonSet(obj *appsv1beta2.DaemonSet) {
 		}
 		if updateStrategy.RollingUpdate.MaxUnavailable == nil {
 			// Set default MaxUnavailable as 1 by default.
-			maxUnavailable := intstr.FromInt32(1)
-			updateStrategy.RollingUpdate.MaxUnavailable = &maxUnavailable
+			updateStrategy.RollingUpdate.MaxUnavailable = ptr.To(intstr.FromInt32(1))
 		}
 		if updateStrategy.RollingUpdate.MaxSurge == nil {
 			// Set default MaxSurge as 0 by default.
-			maxSurge := intstr.FromInt32(0)
-			updateStrategy.RollingUpdate.MaxSurge = &maxSurge
+			updateStrategy.RollingUpdate.MaxSurge = ptr.To(intstr.FromInt32(0))
 		}
 	}
 	if obj.Spec.RevisionHistoryLimit == nil {
@@ -74,12 +72,11 @@ func SetDefaults_StatefulSet(obj *appsv1beta2.StatefulSet) {
 		obj.Spec.UpdateStrategy.RollingUpdate != nil {
 
 		if obj.Spec.UpdateStrategy.RollingUpdate.Partition == nil {
-			obj.Spec.UpdateStrategy.RollingUpdate.Partition = pointer.Int32(0)
+			obj.Spec.UpdateStrategy.RollingUpdate.Partition = ptr.To[int32](0)
 		}
 		if utilfeature.DefaultFeatureGate.Enabled(features.MaxUnavailableStatefulSet) {
 			if obj.Spec.UpdateStrategy.RollingUpdate.MaxUnavailable == nil {
-				maxUnavailable := intstr.FromInt32(1)
-				obj.Spec.UpdateStrategy.RollingUpdate.MaxUnavailable = &maxUnavailable
+				obj.Spec.UpdateStrategy.RollingUpdate.MaxUnavailable = ptr.To(intstr.FromInt32(1))
 			}
 		}
 	}
