@@ -21,10 +21,11 @@ import (
 	"strings"
 	"testing"
 
+	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	api "k8s.io/kubernetes/pkg/apis/core"
 	"k8s.io/kubernetes/pkg/apis/discovery"
-	utilpointer "k8s.io/utils/pointer"
+	"k8s.io/utils/ptr"
 )
 
 func TestValidateEndpointSlice(t *testing.T) {
@@ -43,12 +44,12 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: generateIPAddresses(1),
-					Hostname:  utilpointer.StringPtr("valid-123"),
+					Hostname:  ptr.To("valid-123"),
 				}},
 			},
 		},
@@ -58,12 +59,12 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv6,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: []string{"a00:100::4"},
-					Hostname:  utilpointer.StringPtr("valid-123"),
+					Hostname:  ptr.To("valid-123"),
 				}},
 			},
 		},
@@ -73,12 +74,12 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeFQDN,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: []string{"foo.example.com", "example.com", "example.com.", "hyphens-are-good.example.com"},
-					Hostname:  utilpointer.StringPtr("valid-123"),
+					Hostname:  ptr.To("valid-123"),
 				}},
 			},
 		},
@@ -88,18 +89,18 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("tcp"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("tcp"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}, {
-					Name:     utilpointer.StringPtr("udp"),
-					Protocol: protocolPtr(api.ProtocolUDP),
+					Name:     ptr.To("udp"),
+					Protocol: ptr.To(api.ProtocolUDP),
 				}, {
-					Name:     utilpointer.StringPtr("sctp"),
-					Protocol: protocolPtr(api.ProtocolSCTP),
+					Name:     ptr.To("sctp"),
+					Protocol: ptr.To(api.ProtocolSCTP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: generateIPAddresses(1),
-					Hostname:  utilpointer.StringPtr("valid-123"),
+					Hostname:  ptr.To("valid-123"),
 				}},
 			},
 		},
@@ -109,25 +110,25 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:        utilpointer.StringPtr("one"),
-					Protocol:    protocolPtr(api.ProtocolTCP),
-					AppProtocol: utilpointer.StringPtr("HTTP"),
+					Name:        ptr.To("one"),
+					Protocol:    ptr.To(api.ProtocolTCP),
+					AppProtocol: ptr.To("HTTP"),
 				}, {
-					Name:        utilpointer.StringPtr("two"),
-					Protocol:    protocolPtr(api.ProtocolTCP),
-					AppProtocol: utilpointer.StringPtr("https"),
+					Name:        ptr.To("two"),
+					Protocol:    ptr.To(api.ProtocolTCP),
+					AppProtocol: ptr.To("https"),
 				}, {
-					Name:        utilpointer.StringPtr("three"),
-					Protocol:    protocolPtr(api.ProtocolTCP),
-					AppProtocol: utilpointer.StringPtr("my-protocol"),
+					Name:        ptr.To("three"),
+					Protocol:    ptr.To(api.ProtocolTCP),
+					AppProtocol: ptr.To("my-protocol"),
 				}, {
-					Name:        utilpointer.StringPtr("four"),
-					Protocol:    protocolPtr(api.ProtocolTCP),
-					AppProtocol: utilpointer.StringPtr("example.com/custom-protocol"),
+					Name:        ptr.To("four"),
+					Protocol:    ptr.To(api.ProtocolTCP),
+					AppProtocol: ptr.To("example.com/custom-protocol"),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: generateIPAddresses(1),
-					Hostname:  utilpointer.StringPtr("valid-123"),
+					Hostname:  ptr.To("valid-123"),
 				}},
 			},
 		},
@@ -137,11 +138,11 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr(""),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To(""),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}, {
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: generateIPAddresses(1),
@@ -154,8 +155,8 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr(strings.Repeat("a", 63)),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To(strings.Repeat("a", 63)),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: generateIPAddresses(1),
@@ -194,8 +195,8 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: generateIPAddresses(maxAddresses),
@@ -208,12 +209,29 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
+				}},
+				Endpoints: []discovery.Endpoint{{
+					Addresses:          generateIPAddresses(1),
+					DeprecatedTopology: generateTopology(maxTopologyLabels),
+				}},
+			},
+		},
+		"valid-hints": {
+			expectedErrors: 0,
+			endpointSlice: &discovery.EndpointSlice{
+				ObjectMeta:  standardMeta,
+				AddressType: discovery.AddressTypeIPv4,
+				Ports: []discovery.EndpointPort{{
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: generateIPAddresses(1),
-					Topology:  generateTopology(maxTopologyLabels),
+					Hints: &discovery.EndpointHints{
+						ForZones: []discovery.ForZone{{Name: "zone-a"}},
+					},
 				}},
 			},
 		},
@@ -225,11 +243,11 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr(""),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To(""),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}, {
-					Name:     utilpointer.StringPtr(""),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To(""),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{},
 			},
@@ -240,8 +258,8 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("aCapital"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("aCapital"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{},
 			},
@@ -252,8 +270,8 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("almost_valid"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("almost_valid"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{},
 			},
@@ -264,8 +282,8 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr(strings.Repeat("a", 64)),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To(strings.Repeat("a", 64)),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{},
 			},
@@ -276,8 +294,8 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.Protocol("foo")),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.Protocol("foo")),
 				}},
 			},
 		},
@@ -304,8 +322,8 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: generateIPAddresses(0),
@@ -318,8 +336,8 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: generateIPAddresses(maxAddresses + 1),
@@ -332,12 +350,12 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
-					Addresses: generateIPAddresses(1),
-					Topology:  map[string]string{"--INVALID": "example"},
+					Addresses:          generateIPAddresses(1),
+					DeprecatedTopology: map[string]string{"--INVALID": "example"},
 				}},
 			},
 		},
@@ -347,12 +365,12 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
-					Addresses: generateIPAddresses(1),
-					Topology:  generateTopology(maxTopologyLabels + 1),
+					Addresses:          generateIPAddresses(1),
+					DeprecatedTopology: generateTopology(maxTopologyLabels + 1),
 				}},
 			},
 		},
@@ -362,12 +380,12 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: generateIPAddresses(1),
-					Hostname:  utilpointer.StringPtr("--INVALID"),
+					Hostname:  ptr.To("--INVALID"),
 				}},
 			},
 		},
@@ -380,12 +398,12 @@ func TestValidateEndpointSlice(t *testing.T) {
 				},
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: generateIPAddresses(1),
-					Hostname:  utilpointer.StringPtr("valid-123"),
+					Hostname:  ptr.To("valid-123"),
 				}},
 			},
 		},
@@ -395,12 +413,12 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: []string{"123.456.789.012"},
-					Hostname:  utilpointer.StringPtr("valid-123"),
+					Hostname:  ptr.To("valid-123"),
 				}},
 			},
 		},
@@ -410,12 +428,12 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: []string{"123.456.789.012", "2001:4860:4860::8888"},
-					Hostname:  utilpointer.StringPtr("valid-123"),
+					Hostname:  ptr.To("valid-123"),
 				}},
 			},
 		},
@@ -425,12 +443,12 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv6,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: []string{"123.456.789.012", "2001:4860:4860:defg"},
-					Hostname:  utilpointer.StringPtr("valid-123"),
+					Hostname:  ptr.To("valid-123"),
 				}},
 			},
 		},
@@ -440,12 +458,12 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeFQDN,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: []string{"foo.*", "FOO.example.com", "underscores_are_bad.example.com", "*.example.com"},
-					Hostname:  utilpointer.StringPtr("valid-123"),
+					Hostname:  ptr.To("valid-123"),
 				}},
 			},
 		},
@@ -455,13 +473,78 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:        utilpointer.StringPtr("http"),
-					Protocol:    protocolPtr(api.ProtocolTCP),
-					AppProtocol: utilpointer.StringPtr("--"),
+					Name:        ptr.To("http"),
+					Protocol:    ptr.To(api.ProtocolTCP),
+					AppProtocol: ptr.To("--"),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: generateIPAddresses(1),
-					Hostname:  utilpointer.StringPtr("valid-123"),
+					Hostname:  ptr.To("valid-123"),
+				}},
+			},
+		},
+		"invalid-hints": {
+			expectedErrors: 1,
+			endpointSlice: &discovery.EndpointSlice{
+				ObjectMeta:  standardMeta,
+				AddressType: discovery.AddressTypeIPv4,
+				Ports: []discovery.EndpointPort{{
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
+				}},
+				Endpoints: []discovery.Endpoint{{
+					Addresses: generateIPAddresses(1),
+					Hints: &discovery.EndpointHints{
+						ForZones: []discovery.ForZone{{Name: "inv@lid"}},
+					},
+				}},
+			},
+		},
+		"overlapping-hints": {
+			expectedErrors: 1,
+			endpointSlice: &discovery.EndpointSlice{
+				ObjectMeta:  standardMeta,
+				AddressType: discovery.AddressTypeIPv4,
+				Ports: []discovery.EndpointPort{{
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
+				}},
+				Endpoints: []discovery.Endpoint{{
+					Addresses: generateIPAddresses(1),
+					Hints: &discovery.EndpointHints{
+						ForZones: []discovery.ForZone{
+							{Name: "zone-a"},
+							{Name: "zone-b"},
+							{Name: "zone-a"},
+						},
+					},
+				}},
+			},
+		},
+		"too-many-hints": {
+			expectedErrors: 1,
+			endpointSlice: &discovery.EndpointSlice{
+				ObjectMeta:  standardMeta,
+				AddressType: discovery.AddressTypeIPv4,
+				Ports: []discovery.EndpointPort{{
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
+				}},
+				Endpoints: []discovery.Endpoint{{
+					Addresses: generateIPAddresses(1),
+					Hints: &discovery.EndpointHints{
+						ForZones: []discovery.ForZone{
+							{Name: "zone-a"},
+							{Name: "zone-b"},
+							{Name: "zone-c"},
+							{Name: "zone-d"},
+							{Name: "zone-e"},
+							{Name: "zone-f"},
+							{Name: "zone-g"},
+							{Name: "zone-h"},
+							{Name: "zone-i"},
+						},
+					},
 				}},
 			},
 		},
@@ -469,18 +552,33 @@ func TestValidateEndpointSlice(t *testing.T) {
 			expectedErrors: 3,
 			endpointSlice:  &discovery.EndpointSlice{},
 		},
+		"zone-key-topology": {
+			expectedErrors: 1,
+			endpointSlice: &discovery.EndpointSlice{
+				ObjectMeta:  standardMeta,
+				AddressType: discovery.AddressTypeIPv4,
+				Ports: []discovery.EndpointPort{{
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
+				}},
+				Endpoints: []discovery.Endpoint{{
+					Addresses:          generateIPAddresses(1),
+					DeprecatedTopology: map[string]string{corev1.LabelTopologyZone: "zone1"},
+				}},
+			},
+		},
 		"special-ipv4": {
 			expectedErrors: 1,
 			endpointSlice: &discovery.EndpointSlice{
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: []string{"127.0.0.1"},
-					Hostname:  utilpointer.StringPtr("valid-123"),
+					Hostname:  ptr.To("valid-123"),
 				}},
 			},
 		},
@@ -490,12 +588,12 @@ func TestValidateEndpointSlice(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv6,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: []string{"fe80::9656:d028:8652:66b6"},
-					Hostname:  utilpointer.StringPtr("valid-123"),
+					Hostname:  ptr.To("valid-123"),
 				}},
 			},
 		},
@@ -528,12 +626,12 @@ func TestValidateEndpointSliceCreate(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: generateIPAddresses(1),
-					Hostname:  utilpointer.StringPtr("valid-123"),
+					Hostname:  ptr.To("valid-123"),
 				}},
 			},
 		},
@@ -543,13 +641,13 @@ func TestValidateEndpointSliceCreate(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: generateIPAddresses(1),
-					Hostname:  utilpointer.StringPtr("valid-123"),
-					NodeName:  utilpointer.StringPtr("valid-node-name"),
+					Hostname:  ptr.To("valid-123"),
+					NodeName:  ptr.To("valid-node-name"),
 				}},
 			},
 		},
@@ -561,13 +659,13 @@ func TestValidateEndpointSliceCreate(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: generateIPAddresses(1),
-					Hostname:  utilpointer.StringPtr("valid-123"),
-					NodeName:  utilpointer.StringPtr("INvalid-node-name"),
+					Hostname:  ptr.To("valid-123"),
+					NodeName:  ptr.To("INvalid-node-name"),
 				}},
 			},
 		},
@@ -577,8 +675,8 @@ func TestValidateEndpointSliceCreate(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressType("IP"),
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: generateIPAddresses(1),
@@ -591,8 +689,8 @@ func TestValidateEndpointSliceCreate(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressType("other"),
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr("http"),
-					Protocol: protocolPtr(api.ProtocolTCP),
+					Name:     ptr.To("http"),
+					Protocol: ptr.To(api.ProtocolTCP),
 				}},
 				Endpoints: []discovery.Endpoint{{
 					Addresses: generateIPAddresses(1),
@@ -633,7 +731,7 @@ func TestValidateEndpointSliceUpdate(t *testing.T) {
 		},
 
 		// expected errors
-		"invalide node name set": {
+		"invalid node name set": {
 			oldEndpointSlice: &discovery.EndpointSlice{
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
@@ -646,7 +744,7 @@ func TestValidateEndpointSliceUpdate(t *testing.T) {
 				AddressType: discovery.AddressTypeIPv4,
 				Endpoints: []discovery.Endpoint{{
 					Addresses: []string{"10.1.2.3"},
-					NodeName:  utilpointer.StringPtr("INVALID foo"),
+					NodeName:  ptr.To("INVALID foo"),
 				}},
 			},
 			expectedErrors: 1,
@@ -683,8 +781,8 @@ func TestValidateEndpointSliceUpdate(t *testing.T) {
 				ObjectMeta:  standardMeta,
 				AddressType: discovery.AddressTypeIPv4,
 				Ports: []discovery.EndpointPort{{
-					Name:     utilpointer.StringPtr(""),
-					Protocol: protocolPtr(api.Protocol("invalid")),
+					Name:     ptr.To(""),
+					Protocol: ptr.To(api.Protocol("invalid")),
 				}},
 			},
 			expectedErrors: 1,
@@ -703,16 +801,12 @@ func TestValidateEndpointSliceUpdate(t *testing.T) {
 
 // Test helpers
 
-func protocolPtr(protocol api.Protocol) *api.Protocol {
-	return &protocol
-}
-
 func generatePorts(n int) []discovery.EndpointPort {
 	ports := []discovery.EndpointPort{}
 	for i := 0; i < n; i++ {
 		ports = append(ports, discovery.EndpointPort{
-			Name:     utilpointer.StringPtr(fmt.Sprintf("http-%d", i)),
-			Protocol: protocolPtr(api.ProtocolTCP),
+			Name:     ptr.To(fmt.Sprintf("http-%d", i)),
+			Protocol: ptr.To(api.ProtocolTCP),
 		})
 	}
 	return ports
