@@ -68,7 +68,7 @@ type TokenCleaner struct {
 	// secretSynced returns true if the secret shared informer has been synced at least once.
 	secretSynced cache.InformerSynced
 
-	queue workqueue.RateLimitingInterface
+	queue workqueue.TypedRateLimitingInterface[string]
 }
 
 // NewTokenCleaner returns a new *NewTokenCleaner.
@@ -78,7 +78,12 @@ func NewTokenCleaner(cl clientset.Interface, secrets coreinformers.SecretInforme
 		secretLister:         secrets.Lister(),
 		secretSynced:         secrets.Informer().HasSynced,
 		tokenSecretNamespace: options.TokenSecretNamespace,
-		queue:                workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "token_cleaner"),
+		queue: workqueue.NewTypedRateLimitingQueueWithConfig(
+			workqueue.DefaultTypedControllerRateLimiter[string](),
+			workqueue.TypedRateLimitingQueueConfig[string]{
+				Name: "token_cleaner",
+			},
+		),
 	}
 
 	secrets.Informer().AddEventHandlerWithResyncPeriod(
@@ -108,8 +113,9 @@ func (tc *TokenCleaner) Run(ctx context.Context) {
 	defer utilruntime.HandleCrash()
 	defer tc.queue.ShutDown()
 
-	klog.Infof("Starting token cleaner controller")
-	defer klog.Infof("Shutting down token cleaner controller")
+	logger := klog.FromContext(ctx)
+	logger.Info("Starting token cleaner controller")
+	defer logger.Info("Shutting down token cleaner controller")
 
 	if !cache.WaitForNamedCacheSync("token_cleaner", ctx.Done(), tc.secretSynced) {
 		return
@@ -143,7 +149,7 @@ func (tc *TokenCleaner) processNextWorkItem(ctx context.Context) bool {
 	}
 	defer tc.queue.Done(key)
 
-	if err := tc.syncFunc(ctx, key.(string)); err != nil {
+	if err := tc.syncFunc(ctx, key); err != nil {
 		tc.queue.AddRateLimited(key)
 		utilruntime.HandleError(fmt.Errorf("Sync %v failed with : %v", key, err))
 		return true
@@ -154,9 +160,10 @@ func (tc *TokenCleaner) processNextWorkItem(ctx context.Context) bool {
 }
 
 func (tc *TokenCleaner) syncFunc(ctx context.Context, key string) error {
+	logger := klog.FromContext(ctx)
 	startTime := time.Now()
 	defer func() {
-		klog.V(4).Infof("Finished syncing secret %q (%v)", key, time.Since(startTime))
+		logger.V(4).Info("Finished syncing secret", "secret", key, "elapsedTime", time.Since(startTime))
 	}()
 
 	namespace, name, err := cache.SplitMetaNamespaceKey(key)
@@ -166,7 +173,7 @@ func (tc *TokenCleaner) syncFunc(ctx context.Context, key string) error {
 
 	ret, err := tc.secretLister.Secrets(namespace).Get(name)
 	if apierrors.IsNotFound(err) {
-		klog.V(3).Infof("secret has been deleted: %v", key)
+		logger.V(3).Info("Secret has been deleted", "secret", key)
 		return nil
 	}
 
@@ -181,10 +188,11 @@ func (tc *TokenCleaner) syncFunc(ctx context.Context, key string) error {
 }
 
 func (tc *TokenCleaner) evalSecret(ctx context.Context, o interface{}) {
+	logger := klog.FromContext(ctx)
 	secret := o.(*v1.Secret)
 	ttl, alreadyExpired := bootstrapsecretutil.GetExpiration(secret, time.Now())
 	if alreadyExpired {
-		klog.V(3).Infof("Deleting expired secret %s/%s", secret.Namespace, secret.Name)
+		logger.V(3).Info("Deleting expired secret", "secret", klog.KObj(secret))
 		var options metav1.DeleteOptions
 		if len(secret.UID) > 0 {
 			options.Preconditions = &metav1.Preconditions{UID: &secret.UID}
@@ -193,7 +201,7 @@ func (tc *TokenCleaner) evalSecret(ctx context.Context, o interface{}) {
 		// NotFound isn't a real error (it's already been deleted)
 		// Conflict isn't a real error (the UID precondition failed)
 		if err != nil && !apierrors.IsConflict(err) && !apierrors.IsNotFound(err) {
-			klog.V(3).Infof("Error deleting Secret: %v", err)
+			logger.V(3).Info("Error deleting secret", "err", err)
 		}
 	} else if ttl > 0 {
 		key, err := controller.KeyFunc(o)
