@@ -28,14 +28,12 @@ import (
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/wait"
-	utilfeature "k8s.io/apiserver/pkg/util/feature"
 	corelisters "k8s.io/client-go/listers/core/v1"
 	"k8s.io/client-go/tools/record"
 	"k8s.io/klog/v2"
 	"k8s.io/kubernetes/pkg/controller/volume/attachdetach/cache"
 	"k8s.io/kubernetes/pkg/controller/volume/attachdetach/metrics"
 	"k8s.io/kubernetes/pkg/controller/volume/attachdetach/statusupdater"
-	"k8s.io/kubernetes/pkg/features"
 	kevents "k8s.io/kubernetes/pkg/kubelet/events"
 	"k8s.io/kubernetes/pkg/util/goroutinemap/exponentialbackoff"
 	nodeutil "k8s.io/kubernetes/pkg/util/node"
@@ -71,6 +69,7 @@ func NewReconciler(
 	maxWaitForUnmountDuration time.Duration,
 	syncDuration time.Duration,
 	disableReconciliationSync bool,
+	disableForceDetachOnTimeout bool,
 	desiredStateOfWorld cache.DesiredStateOfWorld,
 	actualStateOfWorld cache.ActualStateOfWorld,
 	attacherDetacher operationexecutor.OperationExecutor,
@@ -78,32 +77,34 @@ func NewReconciler(
 	nodeLister corelisters.NodeLister,
 	recorder record.EventRecorder) Reconciler {
 	return &reconciler{
-		loopPeriod:                loopPeriod,
-		maxWaitForUnmountDuration: maxWaitForUnmountDuration,
-		syncDuration:              syncDuration,
-		disableReconciliationSync: disableReconciliationSync,
-		desiredStateOfWorld:       desiredStateOfWorld,
-		actualStateOfWorld:        actualStateOfWorld,
-		attacherDetacher:          attacherDetacher,
-		nodeStatusUpdater:         nodeStatusUpdater,
-		nodeLister:                nodeLister,
-		timeOfLastSync:            time.Now(),
-		recorder:                  recorder,
+		loopPeriod:                  loopPeriod,
+		maxWaitForUnmountDuration:   maxWaitForUnmountDuration,
+		syncDuration:                syncDuration,
+		disableReconciliationSync:   disableReconciliationSync,
+		disableForceDetachOnTimeout: disableForceDetachOnTimeout,
+		desiredStateOfWorld:         desiredStateOfWorld,
+		actualStateOfWorld:          actualStateOfWorld,
+		attacherDetacher:            attacherDetacher,
+		nodeStatusUpdater:           nodeStatusUpdater,
+		nodeLister:                  nodeLister,
+		timeOfLastSync:              time.Now(),
+		recorder:                    recorder,
 	}
 }
 
 type reconciler struct {
-	loopPeriod                time.Duration
-	maxWaitForUnmountDuration time.Duration
-	syncDuration              time.Duration
-	desiredStateOfWorld       cache.DesiredStateOfWorld
-	actualStateOfWorld        cache.ActualStateOfWorld
-	attacherDetacher          operationexecutor.OperationExecutor
-	nodeStatusUpdater         statusupdater.NodeStatusUpdater
-	nodeLister                corelisters.NodeLister
-	timeOfLastSync            time.Time
-	disableReconciliationSync bool
-	recorder                  record.EventRecorder
+	loopPeriod                  time.Duration
+	maxWaitForUnmountDuration   time.Duration
+	syncDuration                time.Duration
+	desiredStateOfWorld         cache.DesiredStateOfWorld
+	actualStateOfWorld          cache.ActualStateOfWorld
+	attacherDetacher            operationexecutor.OperationExecutor
+	nodeStatusUpdater           statusupdater.NodeStatusUpdater
+	nodeLister                  corelisters.NodeLister
+	timeOfLastSync              time.Time
+	disableReconciliationSync   bool
+	disableForceDetachOnTimeout bool
+	recorder                    record.EventRecorder
 }
 
 func (rc *reconciler) Run(ctx context.Context) {
@@ -143,17 +144,13 @@ func (rc *reconciler) syncStates() {
 	rc.attacherDetacher.VerifyVolumesAreAttached(volumesPerNode, rc.actualStateOfWorld)
 }
 
-// hasOutOfServiceTaint returns true if the node has out-of-service taint present
-// and `NodeOutOfServiceVolumeDetach` feature gate is enabled.
+// hasOutOfServiceTaint returns true if the node has out-of-service taint present.
 func (rc *reconciler) hasOutOfServiceTaint(nodeName types.NodeName) (bool, error) {
-	if utilfeature.DefaultFeatureGate.Enabled(features.NodeOutOfServiceVolumeDetach) {
-		node, err := rc.nodeLister.Get(string(nodeName))
-		if err != nil {
-			return false, err
-		}
-		return taints.TaintKeyExists(node.Spec.Taints, v1.TaintNodeOutOfService), nil
+	node, err := rc.nodeLister.Get(string(nodeName))
+	if err != nil {
+		return false, err
 	}
-	return false, nil
+	return taints.TaintKeyExists(node.Spec.Taints, v1.TaintNodeOutOfService), nil
 }
 
 // nodeIsHealthy returns true if the node looks healthy.
@@ -197,13 +194,13 @@ func (rc *reconciler) reconcile(ctx context.Context) {
 			// Because the detach operation updates the ActualStateOfWorld before
 			// marking itself complete, it's possible for the volume to be removed
 			// from the ActualStateOfWorld between the GetAttachedVolumes() check
-			// and the IsOperationPending() check above.
+			// and the IsOperationSafeToRetry() check above.
 			// Check the ActualStateOfWorld again to avoid issuing an unnecessary
 			// detach.
 			// See https://github.com/kubernetes/kubernetes/issues/93902
 			attachState := rc.actualStateOfWorld.GetAttachState(attachedVolume.VolumeName, attachedVolume.NodeName)
 			if attachState == cache.AttachStateDetached {
-				logger.V(5).Info("Volume detached--skipping", "volume", attachedVolume)
+				logger.V(5).Info("Volume detached--skipping", "node", klog.KRef("", string(attachedVolume.NodeName)), "volumeName", attachedVolume.VolumeName)
 				continue
 			}
 
@@ -213,26 +210,31 @@ func (rc *reconciler) reconcile(ctx context.Context) {
 				logger.Error(err, "Cannot trigger detach because it fails to set detach request time with error")
 				continue
 			}
-			// Check whether timeout has reached the maximum waiting time
-			timeout := elapsedTime > rc.maxWaitForUnmountDuration
+			// Check whether the umount drain timer expired
+			maxWaitForUnmountDurationExpired := elapsedTime > rc.maxWaitForUnmountDuration
 
 			isHealthy, err := rc.nodeIsHealthy(attachedVolume.NodeName)
 			if err != nil {
 				logger.Error(err, "Failed to get health of node", "node", klog.KRef("", string(attachedVolume.NodeName)))
 			}
 
-			// Force detach volumes from unhealthy nodes after maxWaitForUnmountDuration.
-			forceDetach := !isHealthy && timeout
+			// Force detach volumes from unhealthy nodes after maxWaitForUnmountDuration if force detach is enabled
+			// Ensure that the timeout condition checks this correctly so that the correct metric is updated below
+			forceDetatchTimeoutExpired := maxWaitForUnmountDurationExpired && !rc.disableForceDetachOnTimeout
+			if maxWaitForUnmountDurationExpired && rc.disableForceDetachOnTimeout {
+				logger.V(5).Info("Drain timeout expired for volume but disableForceDetachOnTimeout was set", "node", klog.KRef("", string(attachedVolume.NodeName)), "volumeName", attachedVolume.VolumeName)
+			}
+			forceDetach := !isHealthy && forceDetatchTimeoutExpired
 
 			hasOutOfServiceTaint, err := rc.hasOutOfServiceTaint(attachedVolume.NodeName)
 			if err != nil {
 				logger.Error(err, "Failed to get taint specs for node", "node", klog.KRef("", string(attachedVolume.NodeName)))
 			}
 
-			// Check whether volume is still mounted. Skip detach if it is still mounted unless force detach timeout
-			// or the node has `node.kubernetes.io/out-of-service` taint.
+			// Check whether volume is still mounted. Skip detach if it is still mounted unless we have
+			// decided to force detach or the node has `node.kubernetes.io/out-of-service` taint.
 			if attachedVolume.MountedByNode && !forceDetach && !hasOutOfServiceTaint {
-				logger.V(5).Info("Cannot detach volume because it is still mounted", "volume", attachedVolume)
+				logger.V(5).Info("Cannot detach volume because it is still mounted", "node", klog.KRef("", string(attachedVolume.NodeName)), "volumeName", attachedVolume.VolumeName)
 				continue
 			}
 
@@ -252,7 +254,7 @@ func (rc *reconciler) reconcile(ctx context.Context) {
 			err = rc.nodeStatusUpdater.UpdateNodeStatusForNode(logger, attachedVolume.NodeName)
 			if err != nil {
 				// Skip detaching this volume if unable to update node status
-				logger.Error(err, "UpdateNodeStatusForNode failed while attempting to report volume as attached", "volume", attachedVolume)
+				logger.Error(err, "UpdateNodeStatusForNode failed while attempting to report volume as attached", "node", klog.KRef("", string(attachedVolume.NodeName)), "volumeName", attachedVolume.VolumeName)
 				// Add volume back to ReportAsAttached if UpdateNodeStatusForNode call failed so that node status updater will add it back to VolumeAttached list.
 				// It is needed here too because DetachVolume is not call actually and we keep the data consistency for every reconcile.
 				rc.actualStateOfWorld.AddVolumeToReportAsAttached(logger, attachedVolume.VolumeName, attachedVolume.NodeName)
@@ -260,20 +262,30 @@ func (rc *reconciler) reconcile(ctx context.Context) {
 			}
 
 			// Trigger detach volume which requires verifying safe to detach step
-			// If timeout is true, skip verifySafeToDetach check
+			// If forceDetatchTimeoutExpired is true, skip verifySafeToDetach check
 			// If the node has node.kubernetes.io/out-of-service taint with NoExecute effect, skip verifySafeToDetach check
-			logger.V(5).Info("Starting attacherDetacher.DetachVolume", "volume", attachedVolume)
+			logger.V(5).Info("Starting attacherDetacher.DetachVolume", "node", klog.KRef("", string(attachedVolume.NodeName)), "volumeName", attachedVolume.VolumeName)
 			if hasOutOfServiceTaint {
 				logger.V(4).Info("node has out-of-service taint", "node", klog.KRef("", string(attachedVolume.NodeName)))
 			}
-			verifySafeToDetach := !(timeout || hasOutOfServiceTaint)
+			verifySafeToDetach := !(forceDetatchTimeoutExpired || hasOutOfServiceTaint)
 			err = rc.attacherDetacher.DetachVolume(logger, attachedVolume.AttachedVolume, verifySafeToDetach, rc.actualStateOfWorld)
 			if err == nil {
-				if !timeout {
-					logger.Info("attacherDetacher.DetachVolume started", "volume", attachedVolume)
-				} else {
-					metrics.RecordForcedDetachMetric()
-					logger.Info("attacherDetacher.DetachVolume started: this volume is not safe to detach, but maxWaitForUnmountDuration expired, force detaching", "duration", rc.maxWaitForUnmountDuration, "volume", attachedVolume)
+				if verifySafeToDetach { // normal detach
+					logger.Info("attacherDetacher.DetachVolume started", "node", klog.KRef("", string(attachedVolume.NodeName)), "volumeName", attachedVolume.VolumeName)
+				} else { // force detach
+					if forceDetatchTimeoutExpired {
+						metrics.RecordForcedDetachMetric(metrics.ForceDetachReasonTimeout)
+						logger.Info("attacherDetacher.DetachVolume started: this volume is not safe to detach, but maxWaitForUnmountDuration expired, force detaching",
+							"duration", rc.maxWaitForUnmountDuration,
+							"node", klog.KRef("", string(attachedVolume.NodeName)),
+							"volumeName", attachedVolume.VolumeName)
+					} else {
+						metrics.RecordForcedDetachMetric(metrics.ForceDetachReasonOutOfService)
+						logger.Info("attacherDetacher.DetachVolume started: node has out-of-service taint, force detaching",
+							"node", klog.KRef("", string(attachedVolume.NodeName)),
+							"volumeName", attachedVolume.VolumeName)
+					}
 				}
 			}
 			if err != nil {
@@ -285,7 +297,7 @@ func (rc *reconciler) reconcile(ctx context.Context) {
 				if !exponentialbackoff.IsExponentialBackoff(err) {
 					// Ignore exponentialbackoff.IsExponentialBackoff errors, they are expected.
 					// Log all other errors.
-					logger.Error(err, "attacherDetacher.DetachVolume failed to start", "volume", attachedVolume)
+					logger.Error(err, "attacherDetacher.DetachVolume failed to start", "node", klog.KRef("", string(attachedVolume.NodeName)), "volumeName", attachedVolume.VolumeName)
 				}
 			}
 		}
