@@ -25,7 +25,7 @@ import (
 	"time"
 
 	v1 "k8s.io/api/coordination/v1"
-	v1alpha1 "k8s.io/api/coordination/v1alpha1"
+	v1alpha2 "k8s.io/api/coordination/v1alpha2"
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
@@ -42,7 +42,7 @@ func TestReconcileElectionStep(t *testing.T) {
 	tests := []struct {
 		name                    string
 		leaseNN                 types.NamespacedName
-		candidates              []*v1alpha1.LeaseCandidate
+		candidates              []*v1alpha2.LeaseCandidate
 		existingLease           *v1.Lease
 		expectLease             bool
 		expectedHolderIdentity  *string
@@ -55,7 +55,7 @@ func TestReconcileElectionStep(t *testing.T) {
 		{
 			name:                   "no candidates, no lease, noop",
 			leaseNN:                types.NamespacedName{Namespace: "default", Name: "component-A"},
-			candidates:             []*v1alpha1.LeaseCandidate{},
+			candidates:             []*v1alpha2.LeaseCandidate{},
 			existingLease:          nil,
 			expectLease:            false,
 			expectedHolderIdentity: nil,
@@ -66,7 +66,7 @@ func TestReconcileElectionStep(t *testing.T) {
 		{
 			name:                   "no candidates, lease exists. noop, not managed by CLE",
 			leaseNN:                types.NamespacedName{Namespace: "default", Name: "component-A"},
-			candidates:             []*v1alpha1.LeaseCandidate{},
+			candidates:             []*v1alpha2.LeaseCandidate{},
 			existingLease:          &v1.Lease{},
 			expectLease:            false,
 			expectedHolderIdentity: nil,
@@ -77,18 +77,18 @@ func TestReconcileElectionStep(t *testing.T) {
 		{
 			name:    "candidates exist, no existing lease should create lease",
 			leaseNN: types.NamespacedName{Namespace: "default", Name: "component-A"},
-			candidates: []*v1alpha1.LeaseCandidate{
+			candidates: []*v1alpha2.LeaseCandidate{
 				{
 					ObjectMeta: metav1.ObjectMeta{
 						Namespace: "default",
 						Name:      "component-identity-1",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.19.0",
-						BinaryVersion:       "1.19.0",
-						RenewTime:           ptr.To(metav1.NewMicroTime(fakeClock.Now())),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{v1.OldestEmulationVersion},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.19.0",
+						BinaryVersion:    "1.19.0",
+						RenewTime:        ptr.To(metav1.NewMicroTime(fakeClock.Now())),
+						Strategy:         v1.OldestEmulationVersion,
 					},
 				},
 			},
@@ -102,18 +102,18 @@ func TestReconcileElectionStep(t *testing.T) {
 		{
 			name:    "candidates exist, lease exists, unoptimal should set preferredHolder",
 			leaseNN: types.NamespacedName{Namespace: "default", Name: "component-A"},
-			candidates: []*v1alpha1.LeaseCandidate{
+			candidates: []*v1alpha2.LeaseCandidate{
 				{
 					ObjectMeta: metav1.ObjectMeta{
 						Namespace: "default",
 						Name:      "component-identity-1",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.19.0",
-						BinaryVersion:       "1.19.0",
-						RenewTime:           ptr.To(metav1.NewMicroTime(fakeClock.Now())),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{v1.OldestEmulationVersion},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.19.0",
+						BinaryVersion:    "1.19.0",
+						RenewTime:        ptr.To(metav1.NewMicroTime(fakeClock.Now())),
+						Strategy:         v1.OldestEmulationVersion,
 					},
 				},
 				{
@@ -121,12 +121,12 @@ func TestReconcileElectionStep(t *testing.T) {
 						Namespace: "default",
 						Name:      "component-identity-2",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.18.0",
-						BinaryVersion:       "1.18.0",
-						RenewTime:           ptr.To(metav1.NewMicroTime(fakeClock.Now())),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{v1.OldestEmulationVersion},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.18.0",
+						BinaryVersion:    "1.18.0",
+						RenewTime:        ptr.To(metav1.NewMicroTime(fakeClock.Now())),
+						Strategy:         v1.OldestEmulationVersion,
 					},
 				},
 			},
@@ -151,19 +151,19 @@ func TestReconcileElectionStep(t *testing.T) {
 		{
 			name:    "candidates exist, should only elect leader from acked candidates",
 			leaseNN: types.NamespacedName{Namespace: "default", Name: "component-A"},
-			candidates: []*v1alpha1.LeaseCandidate{
+			candidates: []*v1alpha2.LeaseCandidate{
 				{
 					ObjectMeta: metav1.ObjectMeta{
 						Namespace: "default",
 						Name:      "component-identity-1",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.19.0",
-						BinaryVersion:       "1.19.0",
-						PingTime:            ptr.To(metav1.NewMicroTime(fakeClock.Now().Add(-2 * electionDuration))),
-						RenewTime:           ptr.To(metav1.NewMicroTime(fakeClock.Now().Add(-4 * electionDuration))),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{v1.OldestEmulationVersion},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.19.0",
+						BinaryVersion:    "1.19.0",
+						PingTime:         ptr.To(metav1.NewMicroTime(fakeClock.Now().Add(-2 * electionDuration))),
+						RenewTime:        ptr.To(metav1.NewMicroTime(fakeClock.Now().Add(-4 * electionDuration))),
+						Strategy:         v1.OldestEmulationVersion,
 					},
 				},
 				{
@@ -171,13 +171,13 @@ func TestReconcileElectionStep(t *testing.T) {
 						Namespace: "default",
 						Name:      "component-identity-2",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.20.0",
-						BinaryVersion:       "1.20.0",
-						PingTime:            ptr.To(metav1.NewMicroTime(fakeClock.Now())),
-						RenewTime:           ptr.To(metav1.NewMicroTime(fakeClock.Now())),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{v1.OldestEmulationVersion},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.20.0",
+						BinaryVersion:    "1.20.0",
+						PingTime:         ptr.To(metav1.NewMicroTime(fakeClock.Now())),
+						RenewTime:        ptr.To(metav1.NewMicroTime(fakeClock.Now())),
+						Strategy:         v1.OldestEmulationVersion,
 					},
 				},
 			},
@@ -191,18 +191,18 @@ func TestReconcileElectionStep(t *testing.T) {
 		{
 			name:    "candidates exist, lease exists, lease expired",
 			leaseNN: types.NamespacedName{Namespace: "default", Name: "component-A"},
-			candidates: []*v1alpha1.LeaseCandidate{
+			candidates: []*v1alpha2.LeaseCandidate{
 				{
 					ObjectMeta: metav1.ObjectMeta{
 						Namespace: "default",
 						Name:      "component-identity-1",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.19.0",
-						BinaryVersion:       "1.19.0",
-						RenewTime:           ptr.To(metav1.NewMicroTime(fakeClock.Now())),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{v1.OldestEmulationVersion},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.19.0",
+						BinaryVersion:    "1.19.0",
+						RenewTime:        ptr.To(metav1.NewMicroTime(fakeClock.Now())),
+						Strategy:         v1.OldestEmulationVersion,
 					},
 				},
 			},
@@ -226,19 +226,19 @@ func TestReconcileElectionStep(t *testing.T) {
 		{
 			name:    "candidates exist, no acked candidates should return error",
 			leaseNN: types.NamespacedName{Namespace: "default", Name: "component-A"},
-			candidates: []*v1alpha1.LeaseCandidate{
+			candidates: []*v1alpha2.LeaseCandidate{
 				{
 					ObjectMeta: metav1.ObjectMeta{
 						Namespace: "default",
 						Name:      "component-identity-1",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.19.0",
-						BinaryVersion:       "1.19.0",
-						PingTime:            ptr.To(metav1.NewMicroTime(fakeClock.Now().Add(-1 * time.Minute))),
-						RenewTime:           ptr.To(metav1.NewMicroTime(fakeClock.Now().Add(-2 * time.Minute))),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{v1.OldestEmulationVersion},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.19.0",
+						BinaryVersion:    "1.19.0",
+						PingTime:         ptr.To(metav1.NewMicroTime(fakeClock.Now().Add(-1 * time.Minute))),
+						RenewTime:        ptr.To(metav1.NewMicroTime(fakeClock.Now().Add(-2 * time.Minute))),
+						Strategy:         v1.OldestEmulationVersion,
 					},
 				},
 			},
@@ -251,18 +251,18 @@ func TestReconcileElectionStep(t *testing.T) {
 		{
 			name:    "candidates exist, should ping on election",
 			leaseNN: types.NamespacedName{Namespace: "default", Name: "component-A"},
-			candidates: []*v1alpha1.LeaseCandidate{
+			candidates: []*v1alpha2.LeaseCandidate{
 				{
 					ObjectMeta: metav1.ObjectMeta{
 						Namespace: "default",
 						Name:      "component-identity-1",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.19.0",
-						BinaryVersion:       "1.19.0",
-						RenewTime:           ptr.To(metav1.NewMicroTime(fakeClock.Now().Add(-2 * electionDuration))),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{v1.OldestEmulationVersion},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.19.0",
+						BinaryVersion:    "1.19.0",
+						RenewTime:        ptr.To(metav1.NewMicroTime(fakeClock.Now().Add(-2 * electionDuration))),
+						Strategy:         v1.OldestEmulationVersion,
 					},
 				},
 			},
@@ -277,19 +277,19 @@ func TestReconcileElectionStep(t *testing.T) {
 		{
 			name:    "candidate exist, pinged candidate should have until electionDuration until election decision is made",
 			leaseNN: types.NamespacedName{Namespace: "default", Name: "component-A"},
-			candidates: []*v1alpha1.LeaseCandidate{
+			candidates: []*v1alpha2.LeaseCandidate{
 				{
 					ObjectMeta: metav1.ObjectMeta{
 						Namespace: "default",
 						Name:      "component-identity-1",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.19.0",
-						BinaryVersion:       "1.19.0",
-						PingTime:            ptr.To(metav1.NewMicroTime(fakeClock.Now())),
-						RenewTime:           ptr.To(metav1.NewMicroTime(fakeClock.Now().Add(-1 * time.Minute))),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{v1.OldestEmulationVersion},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.19.0",
+						BinaryVersion:    "1.19.0",
+						PingTime:         ptr.To(metav1.NewMicroTime(fakeClock.Now())),
+						RenewTime:        ptr.To(metav1.NewMicroTime(fakeClock.Now().Add(-1 * time.Minute))),
+						Strategy:         v1.OldestEmulationVersion,
 					},
 				},
 			},
@@ -302,18 +302,18 @@ func TestReconcileElectionStep(t *testing.T) {
 		{
 			name:    "candidates exist, lease exists, lease expired, 3rdparty strategy",
 			leaseNN: types.NamespacedName{Namespace: "default", Name: "component-A"},
-			candidates: []*v1alpha1.LeaseCandidate{
+			candidates: []*v1alpha2.LeaseCandidate{
 				{
 					ObjectMeta: metav1.ObjectMeta{
 						Namespace: "default",
 						Name:      "component-identity-1",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.19.0",
-						BinaryVersion:       "1.19.0",
-						RenewTime:           ptr.To(metav1.NewMicroTime(fakeClock.Now())),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{"foo.com/bar"},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.19.0",
+						BinaryVersion:    "1.19.0",
+						RenewTime:        ptr.To(metav1.NewMicroTime(fakeClock.Now())),
+						Strategy:         v1.CoordinatedLeaseStrategy("foo.com/bar"),
 					},
 				},
 			},
@@ -344,9 +344,9 @@ func TestReconcileElectionStep(t *testing.T) {
 
 			controller, err := NewController(
 				informerFactory.Coordination().V1().Leases(),
-				informerFactory.Coordination().V1alpha1().LeaseCandidates(),
+				informerFactory.Coordination().V1alpha2().LeaseCandidates(),
 				client.CoordinationV1(),
-				client.CoordinationV1alpha1(),
+				client.CoordinationV1alpha2(),
 			)
 			controller.clock = fakeClock
 			if err != nil {
@@ -363,7 +363,7 @@ func TestReconcileElectionStep(t *testing.T) {
 
 			// Set up the fake client with the candidates
 			for _, candidate := range tc.candidates {
-				_, err = client.CoordinationV1alpha1().LeaseCandidates(candidate.Namespace).Create(ctx, candidate, metav1.CreateOptions{})
+				_, err = client.CoordinationV1alpha2().LeaseCandidates(candidate.Namespace).Create(ctx, candidate, metav1.CreateOptions{})
 				if err != nil {
 					t.Fatal(err)
 				}
@@ -414,11 +414,11 @@ func TestReconcileElectionStep(t *testing.T) {
 			// Verify that ping to candidate was issued
 			if tc.candidatesPinged {
 				pinged := false
-				candidatesList, err := client.CoordinationV1alpha1().LeaseCandidates(tc.leaseNN.Namespace).List(ctx, metav1.ListOptions{})
+				candidatesList, err := client.CoordinationV1alpha2().LeaseCandidates(tc.leaseNN.Namespace).List(ctx, metav1.ListOptions{})
 				if err != nil {
 					t.Fatal(err)
 				}
-				oldCandidateMap := make(map[string]*v1alpha1.LeaseCandidate)
+				oldCandidateMap := make(map[string]*v1alpha2.LeaseCandidate)
 				for _, candidate := range tc.candidates {
 					oldCandidateMap[candidate.Name] = candidate
 				}
@@ -443,25 +443,25 @@ func TestController(t *testing.T) {
 	cases := []struct {
 		name                            string
 		leases                          []*v1.Lease
-		candidates                      []*v1alpha1.LeaseCandidate
-		createAfterControllerStart      []*v1alpha1.LeaseCandidate
+		candidates                      []*v1alpha2.LeaseCandidate
+		createAfterControllerStart      []*v1alpha2.LeaseCandidate
 		deleteLeaseAfterControllerStart []types.NamespacedName
 		expectedLeaderLeases            []*v1.Lease
 	}{
 		{
 			name: "single candidate leader election",
-			candidates: []*v1alpha1.LeaseCandidate{
+			candidates: []*v1alpha2.LeaseCandidate{
 				{
 					ObjectMeta: metav1.ObjectMeta{
 						Namespace: "kube-system",
 						Name:      "component-identity-1",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.19.0",
-						BinaryVersion:       "1.19.0",
-						RenewTime:           ptr.To(metav1.NewMicroTime(time.Now())),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{v1.OldestEmulationVersion},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.19.0",
+						BinaryVersion:    "1.19.0",
+						RenewTime:        ptr.To(metav1.NewMicroTime(time.Now())),
+						Strategy:         v1.OldestEmulationVersion,
 					},
 				},
 			},
@@ -479,18 +479,18 @@ func TestController(t *testing.T) {
 		},
 		{
 			name: "multiple candidate leader election",
-			candidates: []*v1alpha1.LeaseCandidate{
+			candidates: []*v1alpha2.LeaseCandidate{
 				{
 					ObjectMeta: metav1.ObjectMeta{
 						Namespace: "kube-system",
 						Name:      "component-identity-1",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.19.0",
-						BinaryVersion:       "1.19.0",
-						RenewTime:           ptr.To(metav1.NewMicroTime(time.Now())),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{v1.OldestEmulationVersion},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.19.0",
+						BinaryVersion:    "1.19.0",
+						RenewTime:        ptr.To(metav1.NewMicroTime(time.Now())),
+						Strategy:         v1.OldestEmulationVersion,
 					},
 				},
 				{
@@ -498,12 +498,12 @@ func TestController(t *testing.T) {
 						Namespace: "kube-system",
 						Name:      "component-identity-2",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.19.0",
-						BinaryVersion:       "1.20.0",
-						RenewTime:           ptr.To(metav1.NewMicroTime(time.Now())),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{v1.OldestEmulationVersion},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.19.0",
+						BinaryVersion:    "1.20.0",
+						RenewTime:        ptr.To(metav1.NewMicroTime(time.Now())),
+						Strategy:         v1.OldestEmulationVersion,
 					},
 				},
 				{
@@ -511,12 +511,12 @@ func TestController(t *testing.T) {
 						Namespace: "kube-system",
 						Name:      "component-identity-3",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.20.0",
-						BinaryVersion:       "1.20.0",
-						RenewTime:           ptr.To(metav1.NewMicroTime(time.Now())),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{v1.OldestEmulationVersion},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.20.0",
+						BinaryVersion:    "1.20.0",
+						RenewTime:        ptr.To(metav1.NewMicroTime(time.Now())),
+						Strategy:         v1.OldestEmulationVersion,
 					},
 				},
 			},
@@ -547,18 +547,18 @@ func TestController(t *testing.T) {
 					},
 				},
 			},
-			candidates: []*v1alpha1.LeaseCandidate{
+			candidates: []*v1alpha2.LeaseCandidate{
 				{
 					ObjectMeta: metav1.ObjectMeta{
 						Namespace: "kube-system",
 						Name:      "component-identity-1",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.19.0",
-						BinaryVersion:       "1.19.0",
-						RenewTime:           ptr.To(metav1.NewMicroTime(time.Now())),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{v1.OldestEmulationVersion},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.19.0",
+						BinaryVersion:    "1.19.0",
+						RenewTime:        ptr.To(metav1.NewMicroTime(time.Now())),
+						Strategy:         v1.OldestEmulationVersion,
 					},
 				},
 			},
@@ -593,18 +593,18 @@ func TestController(t *testing.T) {
 					},
 				},
 			},
-			candidates: []*v1alpha1.LeaseCandidate{
+			candidates: []*v1alpha2.LeaseCandidate{
 				{
 					ObjectMeta: metav1.ObjectMeta{
 						Namespace: "kube-system",
 						Name:      "component-identity-1",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.19.0",
-						BinaryVersion:       "1.19.0",
-						RenewTime:           ptr.To(metav1.NewMicroTime(time.Now())),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{v1.OldestEmulationVersion},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.19.0",
+						BinaryVersion:    "1.19.0",
+						RenewTime:        ptr.To(metav1.NewMicroTime(time.Now())),
+						Strategy:         v1.OldestEmulationVersion,
 					},
 				},
 			},
@@ -636,33 +636,33 @@ func TestController(t *testing.T) {
 					},
 				},
 			},
-			candidates: []*v1alpha1.LeaseCandidate{
+			candidates: []*v1alpha2.LeaseCandidate{
 				{
 					ObjectMeta: metav1.ObjectMeta{
 						Namespace: "kube-system",
 						Name:      "component-identity-1",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.20.0",
-						BinaryVersion:       "1.20.0",
-						RenewTime:           ptr.To(metav1.NewMicroTime(time.Now())),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{v1.OldestEmulationVersion},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.20.0",
+						BinaryVersion:    "1.20.0",
+						RenewTime:        ptr.To(metav1.NewMicroTime(time.Now())),
+						Strategy:         v1.OldestEmulationVersion,
 					},
 				},
 			},
-			createAfterControllerStart: []*v1alpha1.LeaseCandidate{
+			createAfterControllerStart: []*v1alpha2.LeaseCandidate{
 				{
 					ObjectMeta: metav1.ObjectMeta{
 						Namespace: "kube-system",
 						Name:      "component-identity-2",
 					},
-					Spec: v1alpha1.LeaseCandidateSpec{
-						LeaseName:           "component-A",
-						EmulationVersion:    "1.19.0",
-						BinaryVersion:       "1.19.0",
-						RenewTime:           ptr.To(metav1.NewMicroTime(time.Now())),
-						PreferredStrategies: []v1.CoordinatedLeaseStrategy{v1.OldestEmulationVersion},
+					Spec: v1alpha2.LeaseCandidateSpec{
+						LeaseName:        "component-A",
+						EmulationVersion: "1.19.0",
+						BinaryVersion:    "1.19.0",
+						RenewTime:        ptr.To(metav1.NewMicroTime(time.Now())),
+						Strategy:         v1.OldestEmulationVersion,
 					},
 				},
 			},
@@ -693,9 +693,9 @@ func TestController(t *testing.T) {
 			informerFactory := informers.NewSharedInformerFactory(client, 0)
 			controller, err := NewController(
 				informerFactory.Coordination().V1().Leases(),
-				informerFactory.Coordination().V1alpha1().LeaseCandidates(),
+				informerFactory.Coordination().V1alpha2().LeaseCandidates(),
 				client.CoordinationV1(),
-				client.CoordinationV1alpha1(),
+				client.CoordinationV1alpha2(),
 			)
 			if err != nil {
 				t.Fatal(err)
@@ -710,7 +710,7 @@ func TestController(t *testing.T) {
 			}
 			for _, obj := range tc.candidates {
 				t.Logf("Pre-creating lease candidate %s/%s", obj.Namespace, obj.Name)
-				_, err := client.CoordinationV1alpha1().LeaseCandidates(obj.Namespace).Create(ctx, obj, metav1.CreateOptions{})
+				_, err := client.CoordinationV1alpha2().LeaseCandidates(obj.Namespace).Create(ctx, obj, metav1.CreateOptions{})
 				if err != nil {
 					t.Fatalf("Error pre-creating lease candidate %s/%s: %v", obj.Namespace, obj.Name, err)
 				}
@@ -748,7 +748,7 @@ func TestController(t *testing.T) {
 							if *ph == *l.Spec.HolderIdentity {
 								continue
 							}
-							if _, err := client.CoordinationV1alpha1().LeaseCandidates(expectedLease.Namespace).Get(ctx, *l.Spec.HolderIdentity, metav1.GetOptions{}); err != nil {
+							if _, err := client.CoordinationV1alpha2().LeaseCandidates(expectedLease.Namespace).Get(ctx, *l.Spec.HolderIdentity, metav1.GetOptions{}); err != nil {
 								continue // only candidate-aware controllers will follow preferredHolder
 							}
 
@@ -772,7 +772,7 @@ func TestController(t *testing.T) {
 					case <-ctx.Done():
 						return
 					case <-ticker.C:
-						cs, err := client.CoordinationV1alpha1().LeaseCandidates("").List(ctx, metav1.ListOptions{})
+						cs, err := client.CoordinationV1alpha2().LeaseCandidates("").List(ctx, metav1.ListOptions{})
 						if err != nil {
 							t.Logf("Error listing lease candidates: %v", err)
 							continue
@@ -781,7 +781,7 @@ func TestController(t *testing.T) {
 							if c.Spec.PingTime != nil && (c.Spec.RenewTime == nil || c.Spec.PingTime.Time.After(c.Spec.RenewTime.Time)) {
 								t.Logf("Answering ping for %s/%s", c.Namespace, c.Name)
 								c.Spec.RenewTime = &metav1.MicroTime{Time: time.Now()}
-								_, err = client.CoordinationV1alpha1().LeaseCandidates(c.Namespace).Update(ctx, &c, metav1.UpdateOptions{})
+								_, err = client.CoordinationV1alpha2().LeaseCandidates(c.Namespace).Update(ctx, &c, metav1.UpdateOptions{})
 								if err != nil {
 									t.Logf("Error updating lease candidate %s/%s: %v", c.Namespace, c.Name, err)
 								}
@@ -793,7 +793,7 @@ func TestController(t *testing.T) {
 
 			for _, obj := range tc.createAfterControllerStart {
 				t.Logf("Post-creating lease candidate %s/%s", obj.Namespace, obj.Name)
-				_, err := client.CoordinationV1alpha1().LeaseCandidates(obj.Namespace).Create(ctx, obj, metav1.CreateOptions{})
+				_, err := client.CoordinationV1alpha2().LeaseCandidates(obj.Namespace).Create(ctx, obj, metav1.CreateOptions{})
 				if err != nil {
 					t.Fatalf("Error post-creating lease candidate %s/%s: %v", obj.Namespace, obj.Name, err)
 				}
