@@ -56,17 +56,15 @@ import (
 	serverstorage "k8s.io/apiserver/pkg/server/storage"
 	etcd3testing "k8s.io/apiserver/pkg/storage/etcd3/testing"
 	"k8s.io/apiserver/pkg/util/openapi"
-	utilversion "k8s.io/apiserver/pkg/util/version"
 	"k8s.io/client-go/discovery"
 	"k8s.io/client-go/informers"
 	"k8s.io/client-go/kubernetes"
 	restclient "k8s.io/client-go/rest"
-	kubeversion "k8s.io/component-base/version"
+	utilversion "k8s.io/component-base/version"
 	aggregatorscheme "k8s.io/kube-aggregator/pkg/apiserver/scheme"
 	netutils "k8s.io/utils/net"
 
 	"k8s.io/kubernetes/pkg/api/legacyscheme"
-	flowcontrolv1bet3 "k8s.io/kubernetes/pkg/apis/flowcontrol/v1beta3"
 	controlplaneapiserver "k8s.io/kubernetes/pkg/controlplane/apiserver"
 	"k8s.io/kubernetes/pkg/controlplane/reconcilers"
 	"k8s.io/kubernetes/pkg/controlplane/storageversionhashdata"
@@ -242,7 +240,7 @@ func TestVersion(t *testing.T) {
 	if err != nil {
 		t.Errorf("unexpected error: %v", err)
 	}
-	expectedInfo := kubeversion.Get()
+	expectedInfo := utilversion.Get()
 	kubeVersion := utilversion.DefaultKubeEffectiveVersion().BinaryVersion()
 	expectedInfo.Major = fmt.Sprintf("%d", kubeVersion.Major())
 	expectedInfo.Minor = fmt.Sprintf("%d", kubeVersion.Minor())
@@ -280,7 +278,7 @@ func TestAPIVersionOfDiscoveryEndpoints(t *testing.T) {
 	}
 	apiVersions := metav1.APIVersions{}
 	assert.NoError(decodeResponse(resp, &apiVersions))
-	assert.Equal(apiVersions.APIVersion, "")
+	assert.Equal("", apiVersions.APIVersion)
 
 	// /api/v1 exists in release-1.1
 	resp, err = http.Get(server.URL + "/api/v1")
@@ -289,7 +287,7 @@ func TestAPIVersionOfDiscoveryEndpoints(t *testing.T) {
 	}
 	resourceList := metav1.APIResourceList{}
 	assert.NoError(decodeResponse(resp, &resourceList))
-	assert.Equal(resourceList.APIVersion, "")
+	assert.Equal("", resourceList.APIVersion)
 
 	// /apis exists in release-1.1
 	resp, err = http.Get(server.URL + "/apis")
@@ -298,7 +296,7 @@ func TestAPIVersionOfDiscoveryEndpoints(t *testing.T) {
 	}
 	groupList := metav1.APIGroupList{}
 	assert.NoError(decodeResponse(resp, &groupList))
-	assert.Equal(groupList.APIVersion, "")
+	assert.Equal("", groupList.APIVersion)
 
 	// /apis/autoscaling doesn't exist in release-1.1, so the APIVersion field
 	// should be non-empty in the results returned by the server.
@@ -308,7 +306,7 @@ func TestAPIVersionOfDiscoveryEndpoints(t *testing.T) {
 	}
 	group := metav1.APIGroup{}
 	assert.NoError(decodeResponse(resp, &group))
-	assert.Equal(group.APIVersion, "v1")
+	assert.Equal("v1", group.APIVersion)
 
 	// apis/autoscaling/v1 doesn't exist in release-1.1, so the APIVersion field
 	// should be non-empty in the results returned by the server.
@@ -319,7 +317,7 @@ func TestAPIVersionOfDiscoveryEndpoints(t *testing.T) {
 	}
 	resourceList = metav1.APIResourceList{}
 	assert.NoError(decodeResponse(resp, &resourceList))
-	assert.Equal(resourceList.APIVersion, "v1")
+	assert.Equal("v1", resourceList.APIVersion)
 
 }
 
@@ -426,14 +424,6 @@ func TestDefaultVars(t *testing.T) {
 		}
 	}
 
-	// legacyBetaEnabledByDefaultResources should contain only beta version
-	for i := range legacyBetaEnabledByDefaultResources {
-		gv := legacyBetaEnabledByDefaultResources[i]
-		if !strings.Contains(gv.Version, "beta") {
-			t.Errorf("legacyBetaEnabledByDefaultResources should contain beta version, but found: %q", gv.String())
-		}
-	}
-
 	// betaAPIGroupVersionsDisabledByDefault should contain only beta version
 	for i := range betaAPIGroupVersionsDisabledByDefault {
 		gv := betaAPIGroupVersionsDisabledByDefault[i]
@@ -466,15 +456,6 @@ func TestNewBetaResourcesEnabledByDefault(t *testing.T) {
 		storageapiv1beta1.SchemeGroupVersion.WithResource("csinodes"):                     true,
 	}
 
-	// legacyBetaResourcesWithoutStableEquivalents contains those groupresources that were enabled by default as beta
-	// before we changed that policy and do not have stable versions. These resources are allowed to have additional
-	// beta versions enabled by default.  Nothing new should be added here.  There are no future exceptions because there
-	// are no more beta resources enabled by default.
-	legacyBetaResourcesWithoutStableEquivalents := map[schema.GroupResource]bool{
-		flowcontrolv1bet3.SchemeGroupVersion.WithResource("flowschemas").GroupResource():                 true,
-		flowcontrolv1bet3.SchemeGroupVersion.WithResource("prioritylevelconfigurations").GroupResource(): true,
-	}
-
 	config := DefaultAPIResourceConfigSource()
 	for gvr, enable := range config.ResourceConfigs {
 		if !strings.Contains(gvr.Version, "beta") {
@@ -486,9 +467,6 @@ func TestNewBetaResourcesEnabledByDefault(t *testing.T) {
 		if legacyEnabledBetaResources[gvr] {
 			continue // this is a legacy beta resource
 		}
-		if legacyBetaResourcesWithoutStableEquivalents[gvr.GroupResource()] {
-			continue // this is another beta of a legacy beta resource with no stable equivalent
-		}
 		t.Errorf("no new beta resources can be enabled by default, see https://github.com/kubernetes/enhancements/blob/0ad0fc8269165ca300d05ca51c7ce190a79976a5/keps/sig-architecture/3136-beta-apis-off-by-default/README.md: %v", gvr)
 	}
 }
