@@ -19,6 +19,11 @@ limitations under the License.
 
 package kuberuntime
 
+import (
+	"k8s.io/kubernetes/pkg/kubelet/cm"
+	"math"
+)
+
 const (
 	milliCPUToCPU = 1000
 
@@ -53,3 +58,22 @@ func milliCPUToQuota(milliCPU int64, period int64) (quota int64) {
 
 	return
 }
+
+// sharesToMilliCPU converts CpuShares (cpu.shares) to milli-CPU value
+// TODO(vinaykul,InPlacePodVerticalScaling): Address issue that sets min req/limit to 2m/10m before beta
+// See: https://github.com/kubernetes/kubernetes/pull/102884#discussion_r662552642
+func sharesToMilliCPU(shares int64) int64 {
+	milliCPU := int64(0)
+	if shares >= int64(cm.MinShares) {
+		milliCPU = int64(math.Ceil(float64(shares*milliCPUToCPU) / float64(cm.SharesPerCPU)))
+	}
+	return milliCPU
+}
+
+// quotaToMilliCPU converts cpu.cfs_quota_us and cpu.cfs_period_us to milli-CPU value
+func quotaToMilliCPU(quota int64, period int64) int64 {
+	if quota == -1 {
+		return int64(0)
+	}
+	return (quota * milliCPUToCPU) / period
+}
