@@ -26,7 +26,6 @@ import (
 	"testing"
 	"time"
 
-	"github.com/golang/mock/gomock"
 	cadvisorfs "github.com/google/cadvisor/fs"
 	cadvisorapiv2 "github.com/google/cadvisor/info/v2"
 	"github.com/stretchr/testify/assert"
@@ -148,11 +147,8 @@ func TestCRIListPodStats(t *testing.T) {
 		podLogStats1 = makeFakeLogStats(6000)
 	)
 
-	mockCtrl := gomock.NewController(t)
-	defer mockCtrl.Finish()
-
 	var (
-		mockCadvisor       = cadvisortest.NewMockInterface(mockCtrl)
+		mockCadvisor       = cadvisortest.NewMockInterface(t)
 		mockRuntimeCache   = new(kubecontainertest.MockRuntimeCache)
 		mockPodManager     = new(kubepodtest.MockManager)
 		resourceAnalyzer   = new(fakeResourceAnalyzer)
@@ -216,13 +212,10 @@ func TestCRIListPodStats(t *testing.T) {
 		filepath.Join(kuberuntime.BuildPodLogsDirectory(testPodLogDirectory, "sandbox1-ns", "sandbox1-name", types.UID("sandbox1-uid")), podLogName1): podLogStats1,
 	}
 
-	ctrl := gomock.NewController(t)
-	defer ctrl.Finish()
-
 	fakeOS := &kubecontainertest.FakeOS{}
 	fakeOS.ReadDirFn = func(path string) ([]os.DirEntry, error) {
 		var dirEntries []os.DirEntry
-		mockDE := kubecontainertest.NewMockDirEntry(ctrl)
+		mockDE := kubecontainertest.NewMockDirEntry(t)
 		switch path {
 		case kuberuntime.BuildPodLogsDirectory(testPodLogDirectory, "sandbox0-ns", "sandbox0-name", types.UID("sandbox0-uid")):
 			mockDE.EXPECT().Name().Return(podLogName0)
@@ -250,7 +243,7 @@ func TestCRIListPodStats(t *testing.T) {
 	stats, err := provider.ListPodStats(ctx)
 	assert := assert.New(t)
 	assert.NoError(err)
-	assert.Equal(4, len(stats))
+	assert.Len(stats, 4)
 
 	podStatsMap := make(map[statsapi.PodReference]statsapi.PodStats)
 	for _, s := range stats {
@@ -259,7 +252,7 @@ func TestCRIListPodStats(t *testing.T) {
 
 	p0 := podStatsMap[statsapi.PodReference{Name: "sandbox0-name", UID: "sandbox0-uid", Namespace: "sandbox0-ns"}]
 	assert.Equal(sandbox0.CreatedAt, p0.StartTime.UnixNano())
-	assert.Equal(2, len(p0.Containers))
+	assert.Len(p0.Containers, 2)
 
 	checkEphemeralStorageStats(assert, p0, ephemeralVolumes, []*runtimeapi.ContainerStats{containerStats0, containerStats1},
 		[]*volume.Metrics{containerLogStats0, containerLogStats1}, podLogStats0)
@@ -288,7 +281,7 @@ func TestCRIListPodStats(t *testing.T) {
 
 	p1 := podStatsMap[statsapi.PodReference{Name: "sandbox1-name", UID: "sandbox1-uid", Namespace: "sandbox1-ns"}]
 	assert.Equal(sandbox1.CreatedAt, p1.StartTime.UnixNano())
-	assert.Equal(1, len(p1.Containers))
+	assert.Len(p1.Containers, 1)
 
 	checkEphemeralStorageStats(assert, p1, ephemeralVolumes, []*runtimeapi.ContainerStats{containerStats2},
 		[]*volume.Metrics{containerLogStats2}, podLogStats1)
@@ -305,7 +298,7 @@ func TestCRIListPodStats(t *testing.T) {
 
 	p2 := podStatsMap[statsapi.PodReference{Name: "sandbox2-name", UID: "sandbox2-uid", Namespace: "sandbox2-ns"}]
 	assert.Equal(sandbox2.CreatedAt, p2.StartTime.UnixNano())
-	assert.Equal(1, len(p2.Containers))
+	assert.Len(p2.Containers, 1)
 
 	checkEphemeralStorageStats(assert, p2, ephemeralVolumes, []*runtimeapi.ContainerStats{containerStats4},
 		[]*volume.Metrics{containerLogStats4}, nil)
@@ -324,7 +317,7 @@ func TestCRIListPodStats(t *testing.T) {
 
 	p3 := podStatsMap[statsapi.PodReference{Name: "sandbox3-name", UID: "sandbox3-uid", Namespace: "sandbox3-ns"}]
 	assert.Equal(sandbox3.CreatedAt, p3.StartTime.UnixNano())
-	assert.Equal(1, len(p3.Containers))
+	assert.Len(p3.Containers, 1)
 
 	c8 := p3.Containers[0]
 	assert.Equal(cName8, c8.Name)
@@ -336,6 +329,10 @@ func TestCRIListPodStats(t *testing.T) {
 }
 
 func TestListPodStatsStrictlyFromCRI(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		// TODO: remove skip once the failing test has been fixed.
+		t.Skip("Skip failing test on Windows.")
+	}
 	ctx := context.Background()
 	var (
 		imageFsMountpoint = "/test/mount/point"
@@ -369,10 +366,8 @@ func TestListPodStatsStrictlyFromCRI(t *testing.T) {
 		podLogStats0 = makeFakeLogStats(5000)
 		podLogStats1 = makeFakeLogStats(6000)
 	)
-	mockCtrl := gomock.NewController(t)
-	defer mockCtrl.Finish()
 	var (
-		mockCadvisor       = cadvisortest.NewMockInterface(mockCtrl)
+		mockCadvisor       = cadvisortest.NewMockInterface(t)
 		mockRuntimeCache   = new(kubecontainertest.MockRuntimeCache)
 		mockPodManager     = new(kubepodtest.MockManager)
 		resourceAnalyzer   = new(fakeResourceAnalyzer)
@@ -439,12 +434,10 @@ func TestListPodStatsStrictlyFromCRI(t *testing.T) {
 		filepath.Join(kuberuntime.BuildPodLogsDirectory(testPodLogDirectory, "sandbox0-ns", "sandbox0-name", types.UID("sandbox0-uid")), podLogName0): podLogStats0,
 		filepath.Join(kuberuntime.BuildPodLogsDirectory(testPodLogDirectory, "sandbox1-ns", "sandbox1-name", types.UID("sandbox1-uid")), podLogName1): podLogStats1,
 	}
-	ctrl := gomock.NewController(t)
-	defer ctrl.Finish()
 	fakeOS := &kubecontainertest.FakeOS{}
 	fakeOS.ReadDirFn = func(path string) ([]os.DirEntry, error) {
 		var dirEntries []os.DirEntry
-		mockDE := kubecontainertest.NewMockDirEntry(ctrl)
+		mockDE := kubecontainertest.NewMockDirEntry(t)
 		switch path {
 		case kuberuntime.BuildPodLogsDirectory(testPodLogDirectory, "sandbox0-ns", "sandbox0-name", types.UID("sandbox0-uid")):
 			mockDE.EXPECT().Name().Return(podLogName0)
@@ -475,14 +468,14 @@ func TestListPodStatsStrictlyFromCRI(t *testing.T) {
 	stats, err := provider.ListPodStats(ctx)
 	assert := assert.New(t)
 	assert.NoError(err)
-	assert.Equal(2, len(stats))
+	assert.Len(stats, 2)
 	podStatsMap := make(map[statsapi.PodReference]statsapi.PodStats)
 	for _, s := range stats {
 		podStatsMap[s.PodRef] = s
 	}
 	p0 := podStatsMap[prf0]
 	assert.Equal(sandbox0.CreatedAt, p0.StartTime.UnixNano())
-	assert.Equal(2, len(p0.Containers))
+	assert.Len(p0.Containers, 2)
 
 	checkEphemeralStorageStats(assert, p0, ephemeralVolumes, []*runtimeapi.ContainerStats{containerStats0, containerStats1},
 		[]*volume.Metrics{containerLogStats0, containerLogStats1}, podLogStats0)
@@ -511,7 +504,7 @@ func TestListPodStatsStrictlyFromCRI(t *testing.T) {
 
 	p1 := podStatsMap[prf1]
 	assert.Equal(sandbox1.CreatedAt, p1.StartTime.UnixNano())
-	assert.Equal(1, len(p1.Containers))
+	assert.Len(p1.Containers, 1)
 
 	checkEphemeralStorageStats(assert, p1, ephemeralVolumes, []*runtimeapi.ContainerStats{containerStats2},
 		[]*volume.Metrics{containerLogStats2}, podLogStats1)
@@ -580,11 +573,8 @@ func TestCRIListPodCPUAndMemoryStats(t *testing.T) {
 		containerStats9 = makeFakeContainerStats(container9, imageFsMountpoint)
 	)
 
-	mockCtrl := gomock.NewController(t)
-	defer mockCtrl.Finish()
-
 	var (
-		mockCadvisor       = cadvisortest.NewMockInterface(mockCtrl)
+		mockCadvisor       = cadvisortest.NewMockInterface(t)
 		mockRuntimeCache   = new(kubecontainertest.MockRuntimeCache)
 		mockPodManager     = new(kubepodtest.MockManager)
 		resourceAnalyzer   = new(fakeResourceAnalyzer)
@@ -647,7 +637,7 @@ func TestCRIListPodCPUAndMemoryStats(t *testing.T) {
 	stats, err := provider.ListPodCPUAndMemoryStats(ctx)
 	assert := assert.New(t)
 	assert.NoError(err)
-	assert.Equal(5, len(stats))
+	assert.Len(stats, 5)
 
 	podStatsMap := make(map[statsapi.PodReference]statsapi.PodStats)
 	for _, s := range stats {
@@ -656,7 +646,7 @@ func TestCRIListPodCPUAndMemoryStats(t *testing.T) {
 
 	p0 := podStatsMap[statsapi.PodReference{Name: "sandbox0-name", UID: "sandbox0-uid", Namespace: "sandbox0-ns"}]
 	assert.Equal(sandbox0.CreatedAt, p0.StartTime.UnixNano())
-	assert.Equal(2, len(p0.Containers))
+	assert.Len(p0.Containers, 2)
 	assert.Nil(p0.EphemeralStorage)
 	assert.Nil(p0.VolumeStats)
 	assert.Nil(p0.Network)
@@ -684,7 +674,7 @@ func TestCRIListPodCPUAndMemoryStats(t *testing.T) {
 
 	p1 := podStatsMap[statsapi.PodReference{Name: "sandbox1-name", UID: "sandbox1-uid", Namespace: "sandbox1-ns"}]
 	assert.Equal(sandbox1.CreatedAt, p1.StartTime.UnixNano())
-	assert.Equal(1, len(p1.Containers))
+	assert.Len(p1.Containers, 1)
 	assert.Nil(p1.EphemeralStorage)
 	assert.Nil(p1.VolumeStats)
 	assert.Nil(p1.Network)
@@ -701,7 +691,7 @@ func TestCRIListPodCPUAndMemoryStats(t *testing.T) {
 
 	p2 := podStatsMap[statsapi.PodReference{Name: "sandbox2-name", UID: "sandbox2-uid", Namespace: "sandbox2-ns"}]
 	assert.Equal(sandbox2.CreatedAt, p2.StartTime.UnixNano())
-	assert.Equal(1, len(p2.Containers))
+	assert.Len(p2.Containers, 1)
 	assert.Nil(p2.EphemeralStorage)
 	assert.Nil(p2.VolumeStats)
 	assert.Nil(p2.Network)
@@ -718,7 +708,7 @@ func TestCRIListPodCPUAndMemoryStats(t *testing.T) {
 
 	p3 := podStatsMap[statsapi.PodReference{Name: "sandbox3-name", UID: "sandbox3-uid", Namespace: "sandbox3-ns"}]
 	assert.Equal(sandbox3.CreatedAt, p3.StartTime.UnixNano())
-	assert.Equal(1, len(p3.Containers))
+	assert.Len(p3.Containers, 1)
 
 	c8 := p3.Containers[0]
 	assert.Equal(cName8, c8.Name)
@@ -729,7 +719,7 @@ func TestCRIListPodCPUAndMemoryStats(t *testing.T) {
 
 	p6 := podStatsMap[statsapi.PodReference{Name: "sandbox6-name", UID: "sandbox6-uid", Namespace: "sandbox6-ns"}]
 	assert.Equal(sandbox6.CreatedAt, p6.StartTime.UnixNano())
-	assert.Equal(1, len(p6.Containers))
+	assert.Len(p6.Containers, 1)
 
 	c9 := p6.Containers[0]
 	assert.Equal(cName9, c9.Name)
@@ -748,11 +738,8 @@ func TestCRIImagesFsStats(t *testing.T) {
 		imageFsUsage      = makeFakeImageFsUsage(imageFsMountpoint)
 	)
 
-	mockCtrl := gomock.NewController(t)
-	defer mockCtrl.Finish()
-
 	var (
-		mockCadvisor       = cadvisortest.NewMockInterface(mockCtrl)
+		mockCadvisor       = cadvisortest.NewMockInterface(t)
 		mockRuntimeCache   = new(kubecontainertest.MockRuntimeCache)
 		mockPodManager     = new(kubepodtest.MockManager)
 		resourceAnalyzer   = new(fakeResourceAnalyzer)
