@@ -21,6 +21,7 @@ import (
 
 	v1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	podutil "k8s.io/kubernetes/pkg/api/v1/pod"
 	"k8s.io/kubernetes/pkg/apis/scheduling"
 )
 
@@ -193,21 +194,11 @@ func IsNodeCriticalPod(pod *v1.Pod) bool {
 	return IsCriticalPod(pod) && (pod.Spec.PriorityClassName == scheduling.SystemNodeCritical)
 }
 
-// IsRestartableInitContainer returns true if the initContainer has
-// ContainerRestartPolicyAlways.
-func IsRestartableInitContainer(initContainer *v1.Container) bool {
-	if initContainer.RestartPolicy == nil {
-		return false
-	}
-
-	return *initContainer.RestartPolicy == v1.ContainerRestartPolicyAlways
-}
-
 // HasRestartableInitContainer returns true if the pod has any restartable init
 // container
 func HasRestartableInitContainer(pod *v1.Pod) bool {
 	for _, container := range pod.Spec.InitContainers {
-		if IsRestartableInitContainer(&container) {
+		if podutil.IsRestartableInitContainer(&container) {
 			return true
 		}
 	}
