@@ -21,7 +21,6 @@ import (
 	"k8s.io/apiserver/pkg/registry/generic"
 	genericregistry "k8s.io/apiserver/pkg/registry/generic/registry"
 	"k8s.io/apiserver/pkg/registry/rest"
-	"k8s.io/apiserver/pkg/storage"
 	api "k8s.io/kubernetes/pkg/apis/core"
 	"k8s.io/kubernetes/pkg/printers"
 	printersinternal "k8s.io/kubernetes/pkg/printers/internalversion"
@@ -37,10 +36,11 @@ type REST struct {
 // NewREST returns a RESTStorage object that will work with ConfigMap objects.
 func NewREST(optsGetter generic.RESTOptionsGetter) (*REST, error) {
 	store := &genericregistry.Store{
-		NewFunc:                  func() runtime.Object { return &api.ConfigMap{} },
-		NewListFunc:              func() runtime.Object { return &api.ConfigMapList{} },
-		PredicateFunc:            configmap.Matcher,
-		DefaultQualifiedResource: api.Resource("configmaps"),
+		NewFunc:                   func() runtime.Object { return &api.ConfigMap{} },
+		NewListFunc:               func() runtime.Object { return &api.ConfigMapList{} },
+		PredicateFunc:             configmap.Matcher,
+		DefaultQualifiedResource:  api.Resource("configmaps"),
+		SingularQualifiedResource: api.Resource("configmap"),
 
 		CreateStrategy: configmap.Strategy,
 		UpdateStrategy: configmap.Strategy,
@@ -51,7 +51,6 @@ func NewREST(optsGetter generic.RESTOptionsGetter) (*REST, error) {
 	options := &generic.StoreOptions{
 		RESTOptions: optsGetter,
 		AttrFunc:    configmap.GetAttrs,
-		TriggerFunc: map[string]storage.IndexerFunc{"metadata.name": configmap.NameTriggerFunc},
 	}
 	if err := store.CompleteWithOptions(options); err != nil {
 		return nil, err
