@@ -21,9 +21,6 @@ import (
 	"fmt"
 
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	utilfeature "k8s.io/apiserver/pkg/util/feature"
-	"k8s.io/kubernetes/pkg/features"
-
 	"k8s.io/apimachinery/pkg/fields"
 	"k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/runtime"
@@ -69,12 +66,11 @@ func (persistentvolumeStrategy) GetResetFields() map[fieldpath.APIVersion]*field
 func (persistentvolumeStrategy) PrepareForCreate(ctx context.Context, obj runtime.Object) {
 	pv := obj.(*api.PersistentVolume)
 	pv.Status = api.PersistentVolumeStatus{}
+	pvutil.DropDisabledSpecFields(&pv.Spec, nil)
 
-	if utilfeature.DefaultFeatureGate.Enabled(features.PersistentVolumeLastPhaseTransitionTime) {
-		pv.Status.Phase = api.VolumePending
-		now := NowFunc()
-		pv.Status.LastPhaseTransitionTime = &now
-	}
+	pv.Status.Phase = api.VolumePending
+	now := NowFunc()
+	pv.Status.LastPhaseTransitionTime = &now
 }
 
 func (persistentvolumeStrategy) Validate(ctx context.Context, obj runtime.Object) field.ErrorList {
@@ -102,6 +98,7 @@ func (persistentvolumeStrategy) PrepareForUpdate(ctx context.Context, obj, old r
 	newPv := obj.(*api.PersistentVolume)
 	oldPv := old.(*api.PersistentVolume)
 	newPv.Status = oldPv.Status
+	pvutil.DropDisabledSpecFields(&newPv.Spec, &oldPv.Spec)
 }
 
 func (persistentvolumeStrategy) ValidateUpdate(ctx context.Context, obj, old runtime.Object) field.ErrorList {
@@ -148,20 +145,17 @@ func (persistentvolumeStatusStrategy) PrepareForUpdate(ctx context.Context, obj,
 	oldPv := old.(*api.PersistentVolume)
 	newPv.Spec = oldPv.Spec
 
-	if utilfeature.DefaultFeatureGate.Enabled(features.PersistentVolumeLastPhaseTransitionTime) {
-		switch {
-		case oldPv.Status.Phase == newPv.Status.Phase && newPv.Status.LastPhaseTransitionTime == nil:
-			// phase didn't change, preserve the existing transition time if set
-			newPv.Status.LastPhaseTransitionTime = oldPv.Status.LastPhaseTransitionTime
+	switch {
+	case oldPv.Status.Phase == newPv.Status.Phase && newPv.Status.LastPhaseTransitionTime == nil:
+		// phase didn't change, preserve the existing transition time if set
+		newPv.Status.LastPhaseTransitionTime = oldPv.Status.LastPhaseTransitionTime
 
-		case oldPv.Status.Phase != newPv.Status.Phase && (newPv.Status.LastPhaseTransitionTime == nil || newPv.Status.LastPhaseTransitionTime.Equal(oldPv.Status.LastPhaseTransitionTime)):
-			// phase changed and client didn't set or didn't change the transition time
-			now := NowFunc()
-			newPv.Status.LastPhaseTransitionTime = &now
-		}
+	case oldPv.Status.Phase != newPv.Status.Phase && (newPv.Status.LastPhaseTransitionTime == nil || newPv.Status.LastPhaseTransitionTime.Equal(oldPv.Status.LastPhaseTransitionTime)):
+		// phase changed and client didn't set or didn't change the transition time
+		now := NowFunc()
+		newPv.Status.LastPhaseTransitionTime = &now
 	}
 
-	pvutil.DropDisabledStatusFields(&oldPv.Status, &newPv.Status)
 }
 
 func (persistentvolumeStatusStrategy) ValidateUpdate(ctx context.Context, obj, old runtime.Object) field.ErrorList {
