@@ -34,8 +34,9 @@ import (
 	serverstorage "k8s.io/apiserver/pkg/server/storage"
 	utilfeature "k8s.io/apiserver/pkg/util/feature"
 	"k8s.io/client-go/kubernetes"
-	networkingv1alpha1client "k8s.io/client-go/kubernetes/typed/networking/v1alpha1"
+	networkingv1beta1client "k8s.io/client-go/kubernetes/typed/networking/v1beta1"
 	policyclient "k8s.io/client-go/kubernetes/typed/policy/v1"
+	"k8s.io/klog/v2"
 	"k8s.io/kubernetes/pkg/api/legacyscheme"
 	api "k8s.io/kubernetes/pkg/apis/core"
 	"k8s.io/kubernetes/pkg/cluster/ports"
@@ -139,8 +140,8 @@ func New(c Config) (*legacyProvider, error) {
 			c.Services.IPRepairInterval,
 			client,
 			c.Informers.Core().V1().Services(),
-			c.Informers.Networking().V1alpha1().ServiceCIDRs(),
-			c.Informers.Networking().V1alpha1().IPAddresses(),
+			c.Informers.Networking().V1beta1().ServiceCIDRs(),
+			c.Informers.Networking().V1beta1().IPAddresses(),
 		).RunUntil
 	}
 
@@ -222,7 +223,7 @@ func (p *legacyProvider) NewRESTStorage(apiResourceConfigSource serverstorage.AP
 			utilfeature.DefaultFeatureGate.Enabled(features.ServiceAccountTokenPodNodeInfo) {
 			nodeGetter = nodeStorage.Node.Store
 		}
-		serviceAccountStorage, err = serviceaccountstore.NewREST(restOptionsGetter, p.ServiceAccountIssuer, p.APIAudiences, p.ServiceAccountMaxExpiration, podStorage.Pod.Store, storage["secrets"].(rest.Getter), nodeGetter, p.ExtendExpiration)
+		serviceAccountStorage, err = serviceaccountstore.NewREST(restOptionsGetter, p.ServiceAccountIssuer, p.APIAudiences, p.ServiceAccountMaxExpiration, podStorage.Pod.Store, storage["secrets"].(rest.Getter), nodeGetter, p.ExtendExpiration, p.IsTokenSignerExternal)
 		if err != nil {
 			return genericapiserver.APIGroupInfo{}, err
 		}
@@ -241,6 +242,9 @@ func (p *legacyProvider) NewRESTStorage(apiResourceConfigSource serverstorage.AP
 			storage[resource+"/eviction"] = podStorage.Eviction
 		}
 		storage[resource+"/ephemeralcontainers"] = podStorage.EphemeralContainers
+		if utilfeature.DefaultFeatureGate.Enabled(features.InPlacePodVerticalScaling) {
+			storage[resource+"/resize"] = podStorage.Resize
+		}
 	}
 	if resource := "bindings"; apiResourceConfigSource.ResourceEnabled(corev1.SchemeGroupVersion.WithResource(resource)) {
 		storage[resource] = podStorage.LegacyBinding
@@ -321,7 +325,7 @@ func (p *legacyProvider) NewRESTStorage(apiResourceConfigSource serverstorage.AP
 func (c *Config) newServiceIPAllocators() (registries rangeRegistries, primaryClusterIPAllocator ipallocator.Interface, clusterIPAllocators map[api.IPFamily]ipallocator.Interface, nodePortAllocator *portallocator.PortAllocator, err error) {
 	clusterIPAllocators = map[api.IPFamily]ipallocator.Interface{}
 
-	serviceStorageConfig, err := c.StorageFactory.NewConfig(api.Resource("services"))
+	serviceStorageConfig, err := c.StorageFactory.NewConfig(api.Resource("services"), &api.Service{})
 	if err != nil {
 		return rangeRegistries{}, nil, nil, nil, err
 	}
@@ -347,19 +351,51 @@ func (c *Config) newServiceIPAllocators() (registries rangeRegistries, primaryCl
 			return rangeRegistries{}, nil, nil, nil, fmt.Errorf("cannot create cluster IP allocator: %v", err)
 		}
 	} else {
-		networkingv1alphaClient, err := networkingv1alpha1client.NewForConfig(c.LoopbackClientConfig)
+		networkingv1beta1Client, err := networkingv1beta1client.NewForConfig(c.LoopbackClientConfig)
 		if err != nil {
 			return rangeRegistries{}, nil, nil, nil, err
 		}
+		var bitmapAllocator ipallocator.Interface
+		if !utilfeature.DefaultFeatureGate.Enabled(features.DisableAllocatorDualWrite) {
+			bitmapAllocator, err = ipallocator.New(&serviceClusterIPRange, func(max int, rangeSpec string, offset int) (allocator.Interface, error) {
+				mem := allocator.NewAllocationMapWithOffset(max, rangeSpec, offset)
+				etcd, err := serviceallocator.NewEtcd(mem, "/ranges/serviceips", serviceStorageConfig.ForResource(api.Resource("serviceipallocations")))
+				if err != nil {
+					return nil, err
+				}
+				// It is possible to start apiserver clusters with the new allocator and dual write enable on new environments.
+				// If this is the case we need to initialize the bitmap or it will fail to allocate IP addresses because
+				// the ResourceVersion of the opaque API object is zero.
+				rangeRegistry, err := etcd.Get()
+				if err != nil {
+					return nil, err
+				}
+				rangeRegistry.Range = serviceClusterIPRange.String()
+				if len(rangeRegistry.ResourceVersion) == 0 {
+					klog.Infof("kube-apiserver started with IP allocator and dual write enabled but bitmap allocator does not exist, recreating it ...")
+					err := etcd.CreateOrUpdate(rangeRegistry)
+					if err != nil {
+						return nil, err
+					}
+				}
+				registries.clusterIP = etcd
+				return etcd, nil
+			})
+			if err != nil {
+				return rangeRegistries{}, nil, nil, nil, fmt.Errorf("cannot create cluster IP allocator: %w", err)
+			}
+
+		}
 		// TODO(aojea) Revisit the initialization of the allocators
 		// since right now it depends on the service-cidr flags and
 		// sets the default IPFamily that may not be coherent with the
 		// existing default ServiceCIDR
 		primaryClusterIPAllocator, err = ipallocator.NewMetaAllocator(
-			networkingv1alphaClient,
-			c.Informers.Networking().V1alpha1().ServiceCIDRs(),
-			c.Informers.Networking().V1alpha1().IPAddresses(),
+			networkingv1beta1Client,
+			c.Informers.Networking().V1beta1().ServiceCIDRs(),
+			c.Informers.Networking().V1beta1().IPAddresses(),
 			netutils.IsIPv6CIDR(&serviceClusterIPRange),
+			bitmapAllocator,
 		)
 		if err != nil {
 			return rangeRegistries{}, nil, nil, nil, fmt.Errorf("cannot create cluster IP allocator: %v", err)
@@ -387,19 +423,51 @@ func (c *Config) newServiceIPAllocators() (registries rangeRegistries, primaryCl
 				return rangeRegistries{}, nil, nil, nil, fmt.Errorf("cannot create cluster secondary IP allocator: %v", err)
 			}
 		} else {
-			networkingv1alphaClient, err := networkingv1alpha1client.NewForConfig(c.LoopbackClientConfig)
+			networkingv1beta1Client, err := networkingv1beta1client.NewForConfig(c.LoopbackClientConfig)
 			if err != nil {
 				return rangeRegistries{}, nil, nil, nil, err
 			}
+			var bitmapAllocator ipallocator.Interface
+			if !utilfeature.DefaultFeatureGate.Enabled(features.DisableAllocatorDualWrite) {
+				bitmapAllocator, err = ipallocator.New(&c.Services.SecondaryClusterIPRange, func(max int, rangeSpec string, offset int) (allocator.Interface, error) {
+					mem := allocator.NewAllocationMapWithOffset(max, rangeSpec, offset)
+					// TODO etcdallocator package to return a storage interface via the storageFactory
+					etcd, err := serviceallocator.NewEtcd(mem, "/ranges/secondaryserviceips", serviceStorageConfig.ForResource(api.Resource("serviceipallocations")))
+					if err != nil {
+						return nil, err
+					}
+					// It is possible to start apiserver clusters with the new allocator and dual write enable on new environments.
+					// If this is the case we need to initialize the bitmap or it will fail to allocate IP addresses because
+					// the ResourceVersion of the opaque API object is zero.
+					rangeRegistry, err := etcd.Get()
+					if err != nil {
+						return nil, err
+					}
+					rangeRegistry.Range = c.Services.SecondaryClusterIPRange.String()
+					if len(rangeRegistry.ResourceVersion) == 0 {
+						klog.Infof("kube-apiserver started with IP allocator and dual write enabled but bitmap allocator does not exist, recreating it ...")
+						err := etcd.CreateOrUpdate(rangeRegistry)
+						if err != nil {
+							return nil, err
+						}
+					}
+					registries.secondaryClusterIP = etcd
+					return etcd, nil
+				})
+				if err != nil {
+					return rangeRegistries{}, nil, nil, nil, fmt.Errorf("cannot create cluster secondary IP allocator: %w", err)
+				}
+			}
 			// TODO(aojea) Revisit the initialization of the allocators
 			// since right now it depends on the service-cidr flags and
 			// sets the default IPFamily that may not be coherent with the
 			// existing default ServiceCIDR
 			secondaryClusterIPAllocator, err = ipallocator.NewMetaAllocator(
-				networkingv1alphaClient,
-				c.Informers.Networking().V1alpha1().ServiceCIDRs(),
-				c.Informers.Networking().V1alpha1().IPAddresses(),
+				networkingv1beta1Client,
+				c.Informers.Networking().V1beta1().ServiceCIDRs(),
+				c.Informers.Networking().V1beta1().IPAddresses(),
 				netutils.IsIPv6CIDR(&c.Services.SecondaryClusterIPRange),
+				bitmapAllocator,
 			)
 			if err != nil {
 				return rangeRegistries{}, nil, nil, nil, fmt.Errorf("cannot create cluster secondary IP allocator: %v", err)
@@ -449,7 +517,7 @@ func (p *legacyProvider) PostStartHook() (string, genericapiserver.PostStartHook
 		runner.Start()
 		go func() {
 			defer runner.Stop()
-			<-context.StopCh
+			<-context.Done()
 		}()
 
 		// For backward compatibility, we ensure that if we never are able
