@@ -26,7 +26,9 @@ import (
 	"unicode"
 
 	restful "github.com/emicklei/go-restful/v3"
-	apidiscoveryv2beta1 "k8s.io/api/apidiscovery/v2beta1"
+	"sigs.k8s.io/structured-merge-diff/v4/fieldpath"
+
+	apidiscoveryv2 "k8s.io/api/apidiscovery/v2"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/conversion"
 	"k8s.io/apimachinery/pkg/runtime"
@@ -46,12 +48,12 @@ import (
 	"k8s.io/apiserver/pkg/storageversion"
 	utilfeature "k8s.io/apiserver/pkg/util/feature"
 	versioninfo "k8s.io/component-base/version"
-	"sigs.k8s.io/structured-merge-diff/v4/fieldpath"
 )
 
 const (
-	ROUTE_META_GVK    = "x-kubernetes-group-version-kind"
-	ROUTE_META_ACTION = "x-kubernetes-action"
+	RouteMetaGVK              = "x-kubernetes-group-version-kind"
+	RouteMetaSelectableFields = "x-kubernetes-selectable-fields"
+	RouteMetaAction           = "x-kubernetes-action"
 )
 
 type APIInstaller struct {
@@ -69,8 +71,8 @@ type action struct {
 	AllNamespaces bool // true iff the action is namespaced but works on aggregate result for all namespaces
 }
 
-func ConvertGroupVersionIntoToDiscovery(list []metav1.APIResource) ([]apidiscoveryv2beta1.APIResourceDiscovery, error) {
-	var apiResourceList []apidiscoveryv2beta1.APIResourceDiscovery
+func ConvertGroupVersionIntoToDiscovery(list []metav1.APIResource) ([]apidiscoveryv2.APIResourceDiscovery, error) {
+	var apiResourceList []apidiscoveryv2.APIResourceDiscovery
 	parentResources := make(map[string]int)
 
 	// Loop through all top-level resources
@@ -80,14 +82,14 @@ func ConvertGroupVersionIntoToDiscovery(list []metav1.APIResource) ([]apidiscove
 			continue
 		}
 
-		var scope apidiscoveryv2beta1.ResourceScope
+		var scope apidiscoveryv2.ResourceScope
 		if r.Namespaced {
-			scope = apidiscoveryv2beta1.ScopeNamespace
+			scope = apidiscoveryv2.ScopeNamespace
 		} else {
-			scope = apidiscoveryv2beta1.ScopeCluster
+			scope = apidiscoveryv2.ScopeCluster
 		}
 
-		resource := apidiscoveryv2beta1.APIResourceDiscovery{
+		resource := apidiscoveryv2.APIResourceDiscovery{
 			Resource: r.Name,
 			Scope:    scope,
 			ResponseKind: &metav1.GroupVersionKind{
@@ -114,17 +116,17 @@ func ConvertGroupVersionIntoToDiscovery(list []metav1.APIResource) ([]apidiscove
 			continue
 		}
 
-		var scope apidiscoveryv2beta1.ResourceScope
+		var scope apidiscoveryv2.ResourceScope
 		if r.Namespaced {
-			scope = apidiscoveryv2beta1.ScopeNamespace
+			scope = apidiscoveryv2.ScopeNamespace
 		} else {
-			scope = apidiscoveryv2beta1.ScopeCluster
+			scope = apidiscoveryv2.ScopeCluster
 		}
 
 		parentidx, exists := parentResources[split[0]]
 		if !exists {
 			// If a subresource exists without a parent, create a parent
-			apiResourceList = append(apiResourceList, apidiscoveryv2beta1.APIResourceDiscovery{
+			apiResourceList = append(apiResourceList, apidiscoveryv2.APIResourceDiscovery{
 				Resource: split[0],
 				Scope:    scope,
 				// avoid nil panics in v0.26.0-v0.26.3 client-go clients
@@ -140,7 +142,7 @@ func ConvertGroupVersionIntoToDiscovery(list []metav1.APIResource) ([]apidiscove
 			//
 		}
 
-		subresource := apidiscoveryv2beta1.APISubresourceDiscovery{
+		subresource := apidiscoveryv2.APISubresourceDiscovery{
 			Subresource: split[1],
 			Verbs:       r.Verbs,
 			// avoid nil panics in v0.26.0-v0.26.3 client-go clients
@@ -683,9 +685,27 @@ func (a *APIInstaller) registerResourceHandlers(path string, storage rest.Storag
 		reqScope.MetaGroupVersion = *a.group.MetaGroupVersion
 	}
 
-	var resetFields map[fieldpath.APIVersion]*fieldpath.Set
-	if resetFieldsStrategy, isResetFieldsStrategy := storage.(rest.ResetFieldsStrategy); isResetFieldsStrategy {
-		resetFields = resetFieldsStrategy.GetResetFields()
+	// Strategies may ignore changes to some fields by resetting the field values.
+	//
+	// For instance, spec resource strategies should reset the status, and status subresource
+	// strategies should reset the spec.
+	//
+	// Strategies that reset fields must report to the field manager which fields are
+	// reset by implementing either the ResetFieldsStrategy or the ResetFieldsFilterStrategy
+	// interface.
+	//
+	// For subresources that provide write access to only specific nested fields
+	// fieldpath.NewPatternFilter can help create a filter to reset all other fields.
+	var resetFieldsFilter map[fieldpath.APIVersion]fieldpath.Filter
+	resetFieldsStrategy, isResetFieldsStrategy := storage.(rest.ResetFieldsStrategy)
+	if isResetFieldsStrategy {
+		resetFieldsFilter = fieldpath.NewExcludeFilterSetMap(resetFieldsStrategy.GetResetFields())
+	}
+	if resetFieldsStrategy, isResetFieldsFilterStrategy := storage.(rest.ResetFieldsFilterStrategy); isResetFieldsFilterStrategy {
+		if isResetFieldsStrategy {
+			return nil, nil, fmt.Errorf("may not implement both ResetFieldsStrategy and ResetFieldsFilterStrategy")
+		}
+		resetFieldsFilter = resetFieldsStrategy.GetResetFieldsFilter()
 	}
 
 	reqScope.FieldManager, err = managedfields.NewDefaultFieldManager(
@@ -696,7 +716,7 @@ func (a *APIInstaller) registerResourceHandlers(path string, storage rest.Storag
 		fqKindToRegister,
 		reqScope.HubGroupVersion,
 		subresource,
-		resetFields,
+		resetFieldsFilter,
 	)
 	if err != nil {
 		return nil, nil, fmt.Errorf("failed to create field manager: %v", err)
@@ -873,7 +893,10 @@ func (a *APIInstaller) registerResourceHandlers(path string, storage rest.Storag
 				string(types.JSONPatchType),
 				string(types.MergePatchType),
 				string(types.StrategicMergePatchType),
-				string(types.ApplyPatchType),
+				string(types.ApplyYAMLPatchType),
+			}
+			if utilfeature.DefaultFeatureGate.Enabled(features.CBORServingAndStorage) {
+				supportedTypes = append(supportedTypes, string(types.ApplyCBORPatchType))
 			}
 			handler := metrics.InstrumentRouteFunc(action.Verb, group, version, resource, subresource, requestScope, metrics.APIServerComponent, deprecated, removedRelease, restfulPatchResource(patcher, reqScope, admit, supportedTypes))
 			handler = utilwarning.AddWarningsHandler(handler, warnings)
@@ -1059,12 +1082,12 @@ func (a *APIInstaller) registerResourceHandlers(path string, storage rest.Storag
 			return nil, nil, fmt.Errorf("unrecognized action verb: %s", action.Verb)
 		}
 		for _, route := range routes {
-			route.Metadata(ROUTE_META_GVK, metav1.GroupVersionKind{
+			route.Metadata(RouteMetaGVK, metav1.GroupVersionKind{
 				Group:   reqScope.Kind.Group,
 				Version: reqScope.Kind.Version,
 				Kind:    reqScope.Kind.Kind,
 			})
-			route.Metadata(ROUTE_META_ACTION, strings.ToLower(action.Verb))
+			route.Metadata(RouteMetaAction, strings.ToLower(action.Verb))
 			ws.Route(route)
 		}
 		// Note: update GetAuthorizerAttributes() when adding a custom handler.
@@ -1193,6 +1216,8 @@ func typeToJSON(typeName string) string {
 		return "string"
 	case "v1.IncludeObjectPolicy", "*v1.IncludeObjectPolicy":
 		return "string"
+	case "*string":
+		return "string"
 
 	// TODO: Fix these when go-restful supports a way to specify an array query param:
 	// https://github.com/emicklei/go-restful/issues/225
