@@ -197,9 +197,9 @@ func TestStorageSizeCollector(t *testing.T) {
 				return []Monitor{fakeEtcdMonitor{storageSize: 1e9}}, nil
 			},
 			err: nil,
-			want: `# HELP apiserver_storage_size_bytes [ALPHA] Size of the storage database file physically allocated in bytes.
+			want: `# HELP apiserver_storage_size_bytes [STABLE] Size of the storage database file physically allocated in bytes.
 			# TYPE apiserver_storage_size_bytes gauge
-			apiserver_storage_size_bytes{cluster="etcd-0"} 1e+09
+			apiserver_storage_size_bytes{storage_cluster_id="etcd-0"} 1e+09
 			`,
 		},
 		{
@@ -226,6 +226,48 @@ func TestStorageSizeCollector(t *testing.T) {
 
 }
 
+func TestUpdateObjectCount(t *testing.T) {
+	registry := metrics.NewKubeRegistry()
+	registry.Register(objectCounts)
+	testedMetrics := "apiserver_storage_objects"
+
+	testCases := []struct {
+		desc     string
+		resource string
+		count    int64
+		want     string
+	}{
+		{
+			desc:     "successful fetch",
+			resource: "foo",
+			count:    10,
+			want: `# HELP apiserver_storage_objects [STABLE] Number of stored objects at the time of last check split by kind. In case of a fetching error, the value will be -1.
+# TYPE apiserver_storage_objects gauge
+apiserver_storage_objects{resource="foo"} 10
+`,
+		},
+		{
+			desc:     "failed fetch",
+			resource: "bar",
+			count:    -1,
+			want: `# HELP apiserver_storage_objects [STABLE] Number of stored objects at the time of last check split by kind. In case of a fetching error, the value will be -1.
+# TYPE apiserver_storage_objects gauge
+apiserver_storage_objects{resource="bar"} -1
+`,
+		},
+	}
+
+	for _, test := range testCases {
+		t.Run(test.desc, func(t *testing.T) {
+			defer registry.Reset()
+			UpdateObjectCount(test.resource, test.count)
+			if err := testutil.GatherAndCompare(registry, strings.NewReader(test.want), testedMetrics); err != nil {
+				t.Fatal(err)
+			}
+		})
+	}
+}
+
 type fakeEtcdMonitor struct {
 	storageSize int64
 }
