@@ -152,7 +152,7 @@ func TestGCMUnsafeCompatibility(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	transformerDecrypt := newGCMTransformer(t, block)
+	transformerDecrypt := newGCMTransformer(t, block, nil)
 
 	ctx := context.Background()
 	dataCtx := value.DefaultContext("authenticated_data")
@@ -184,7 +184,7 @@ func TestGCMLegacyDataCompatibility(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	transformerDecrypt := newGCMTransformer(t, block)
+	transformerDecrypt := newGCMTransformer(t, block, nil)
 
 	// recorded output from NewGCMTransformer at commit 3b1fc60d8010dd8b53e97ba80e4710dbb430beee
 	const legacyCiphertext = "\x9f'\xc8\xfc\xea\x8aX\xc4g\xd8\xe47\xdb\xf2\xd8YU\xf9\xb4\xbd\x91/N\xf9g\u05c8\xa0\xcb\ay}\xac\n?\n\bE`\\\xa8Z\xc8V+J\xe1"
@@ -204,12 +204,36 @@ func TestGCMLegacyDataCompatibility(t *testing.T) {
 	}
 }
 
+func TestExtendedNonceGCMLegacyDataCompatibility(t *testing.T) {
+	// recorded output from NewKDFExtendedNonceGCMTransformerWithUniqueSeed from https://github.com/kubernetes/kubernetes/pull/118828
+	const (
+		legacyKey        = "]@2:\x82\x0f\xf9Uag^;\x95\xe8\x18g\xc5\xfd\xd5a\xd3Z\x88\xa2Ћ\b\xaa\x9dO\xcf\\"
+		legacyCiphertext = "$Bu\x9e3\x94_\xba\xd7\t\xdbWz\x0f\x03\x7fا\t\xfcv\x97\x9b\x89B \x9d\xeb\xce˝W\xef\xe3\xd6\xffj\x1e\xf6\xee\x9aP\x03\xb9\x83;0C\xce\xc1\xe4{5\x17[\x15\x11\a\xa8\xd2Ak\x0e)k\xbff\xb5\xd1\x02\xfc\xefߚx\xf2\x93\xd2q"
+	)
+
+	transformerDecrypt := newHKDFExtendedNonceGCMTransformerTest(t, nil, []byte(legacyKey))
+
+	ctx := context.Background()
+	dataCtx := value.DefaultContext("bamboo")
+
+	plaintext := []byte("pandas are the best")
+
+	plaintextAgain, _, err := transformerDecrypt.TransformFromStorage(ctx, []byte(legacyCiphertext), dataCtx)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	if !bytes.Equal(plaintext, plaintextAgain) {
+		t.Errorf("expected original plaintext %q, got %q", string(plaintext), string(plaintextAgain))
+	}
+}
+
 func TestGCMUnsafeNonceGen(t *testing.T) {
 	block, err := aes.NewCipher([]byte("abcdefghijklmnop"))
 	if err != nil {
 		t.Fatal(err)
 	}
-	transformer := newGCMTransformerWithUniqueKeyUnsafeTest(t, block)
+	transformer := newGCMTransformerWithUniqueKeyUnsafeTest(t, block, nil)
 
 	ctx := context.Background()
 	dataCtx := value.DefaultContext("authenticated_data")
@@ -270,7 +294,7 @@ func TestGCMUnsafeNonceGen(t *testing.T) {
 
 func TestGCMNonce(t *testing.T) {
 	t.Run("gcm", func(t *testing.T) {
-		testGCMNonce(t, newGCMTransformer, func(_ int, nonce []byte) {
+		testGCMNonce(t, newGCMTransformer, 0, func(_ int, nonce []byte) {
 			if bytes.Equal(nonce, make([]byte, len(nonce))) {
 				t.Error("got all zeros for nonce")
 			}
@@ -278,21 +302,30 @@ func TestGCMNonce(t *testing.T) {
 	})
 
 	t.Run("gcm unsafe", func(t *testing.T) {
-		testGCMNonce(t, newGCMTransformerWithUniqueKeyUnsafeTest, func(i int, nonce []byte) {
+		testGCMNonce(t, newGCMTransformerWithUniqueKeyUnsafeTest, 0, func(i int, nonce []byte) {
 			counter := binary.LittleEndian.Uint64(nonce)
 			if uint64(i+1) != counter { // add one because the counter starts at 1, not 0
 				t.Errorf("counter nonce is invalid: want %d, got %d", i+1, counter)
 			}
 		})
 	})
+
+	t.Run("gcm extended nonce", func(t *testing.T) {
+		testGCMNonce(t, newHKDFExtendedNonceGCMTransformerTest, infoSizeExtendedNonceGCM, func(_ int, nonce []byte) {
+			if bytes.Equal(nonce, make([]byte, len(nonce))) {
+				t.Error("got all zeros for nonce")
+			}
+		})
+	})
 }
 
-func testGCMNonce(t *testing.T, f func(t testingT, block cipher.Block) value.Transformer, check func(int, []byte)) {
-	block, err := aes.NewCipher([]byte("abcdefghijklmnop"))
+func testGCMNonce(t *testing.T, f transformerFunc, infoLen int, check func(int, []byte)) {
+	key := []byte("abcdefghijklmnopabcdefghijklmnop")
+	block, err := aes.NewCipher(key)
 	if err != nil {
 		t.Fatal(err)
 	}
-	transformer := f(t, block)
+	transformer := f(t, block, key)
 
 	ctx := context.Background()
 	dataCtx := value.DefaultContext("authenticated_data")
@@ -307,13 +340,20 @@ func testGCMNonce(t *testing.T, f func(t testingT, block cipher.Block) value.Tra
 			t.Fatal(err)
 		}
 
-		nonce := out[:12]
+		info := out[:infoLen]
+		nonce := out[infoLen : 12+infoLen]
 		randomN := nonce[:4]
 
 		if bytes.Equal(randomN, make([]byte, len(randomN))) {
 			t.Error("got all zeros for first four bytes")
 		}
 
+		if infoLen != 0 {
+			if bytes.Equal(info, make([]byte, infoLen)) {
+				t.Error("got all zeros for info")
+			}
+		}
+
 		check(i, nonce[4:])
 	}
 }
@@ -326,15 +366,22 @@ func TestGCMKeyRotation(t *testing.T) {
 	t.Run("gcm unsafe", func(t *testing.T) {
 		testGCMKeyRotation(t, newGCMTransformerWithUniqueKeyUnsafeTest)
 	})
+
+	t.Run("gcm extended", func(t *testing.T) {
+		testGCMKeyRotation(t, newHKDFExtendedNonceGCMTransformerTest)
+	})
 }
 
-func testGCMKeyRotation(t *testing.T, f func(t testingT, block cipher.Block) value.Transformer) {
+func testGCMKeyRotation(t *testing.T, f transformerFunc) {
+	key1 := []byte("abcdefghijklmnopabcdefghijklmnop")
+	key2 := []byte("0123456789abcdef0123456789abcdef")
+
 	testErr := fmt.Errorf("test error")
-	block1, err := aes.NewCipher([]byte("abcdefghijklmnop"))
+	block1, err := aes.NewCipher(key1)
 	if err != nil {
 		t.Fatal(err)
 	}
-	block2, err := aes.NewCipher([]byte("0123456789abcdef"))
+	block2, err := aes.NewCipher(key2)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -343,8 +390,8 @@ func testGCMKeyRotation(t *testing.T, f func(t testingT, block cipher.Block) val
 	dataCtx := value.DefaultContext("authenticated_data")
 
 	p := value.NewPrefixTransformers(testErr,
-		value.PrefixTransformer{Prefix: []byte("first:"), Transformer: f(t, block1)},
-		value.PrefixTransformer{Prefix: []byte("second:"), Transformer: f(t, block2)},
+		value.PrefixTransformer{Prefix: []byte("first:"), Transformer: f(t, block1, key1)},
+		value.PrefixTransformer{Prefix: []byte("second:"), Transformer: f(t, block2, key2)},
 	)
 	out, err := p.TransformToStorage(ctx, []byte("firstvalue"), dataCtx)
 	if err != nil {
@@ -369,8 +416,8 @@ func testGCMKeyRotation(t *testing.T, f func(t testingT, block cipher.Block) val
 
 	// reverse the order, use the second key
 	p = value.NewPrefixTransformers(testErr,
-		value.PrefixTransformer{Prefix: []byte("second:"), Transformer: f(t, block2)},
-		value.PrefixTransformer{Prefix: []byte("first:"), Transformer: f(t, block1)},
+		value.PrefixTransformer{Prefix: []byte("second:"), Transformer: f(t, block2, key2)},
+		value.PrefixTransformer{Prefix: []byte("first:"), Transformer: f(t, block1, key1)},
 	)
 	from, stale, err = p.TransformFromStorage(ctx, out, dataCtx)
 	if err != nil {
@@ -434,6 +481,12 @@ func TestCBCKeyRotation(t *testing.T) {
 	}
 }
 
+var gcmBenchmarks = []namedTransformerFunc{
+	{name: "gcm-random-nonce", f: newGCMTransformer},
+	{name: "gcm-counter-nonce", f: newGCMTransformerWithUniqueKeyUnsafeTest},
+	{name: "gcm-extended-nonce", f: newHKDFExtendedNonceGCMTransformerTest},
+}
+
 func BenchmarkGCMRead(b *testing.B) {
 	tests := []struct {
 		keyLength   int
@@ -448,7 +501,16 @@ func BenchmarkGCMRead(b *testing.B) {
 	for _, t := range tests {
 		name := fmt.Sprintf("%vKeyLength/%vValueLength/%vExpectStale", t.keyLength, t.valueLength, t.expectStale)
 		b.Run(name, func(b *testing.B) {
-			benchmarkGCMRead(b, t.keyLength, t.valueLength, t.expectStale)
+			for _, n := range gcmBenchmarks {
+				n := n
+				if t.keyLength == 16 && n.name == "gcm-extended-nonce" {
+					continue // gcm-extended-nonce requires 32 byte keys
+				}
+				b.Run(n.name, func(b *testing.B) {
+					b.ReportAllocs()
+					benchmarkGCMRead(b, n.f, t.keyLength, t.valueLength, t.expectStale)
+				})
+			}
 		})
 	}
 }
@@ -465,23 +527,35 @@ func BenchmarkGCMWrite(b *testing.B) {
 	for _, t := range tests {
 		name := fmt.Sprintf("%vKeyLength/%vValueLength", t.keyLength, t.valueLength)
 		b.Run(name, func(b *testing.B) {
-			benchmarkGCMWrite(b, t.keyLength, t.valueLength)
+			for _, n := range gcmBenchmarks {
+				n := n
+				if t.keyLength == 16 && n.name == "gcm-extended-nonce" {
+					continue // gcm-extended-nonce requires 32 byte keys
+				}
+				b.Run(n.name, func(b *testing.B) {
+					b.ReportAllocs()
+					benchmarkGCMWrite(b, n.f, t.keyLength, t.valueLength)
+				})
+			}
 		})
 	}
 }
 
-func benchmarkGCMRead(b *testing.B, keyLength int, valueLength int, expectStale bool) {
-	block1, err := aes.NewCipher(bytes.Repeat([]byte("a"), keyLength))
+func benchmarkGCMRead(b *testing.B, f transformerFunc, keyLength int, valueLength int, expectStale bool) {
+	key1 := bytes.Repeat([]byte("a"), keyLength)
+	key2 := bytes.Repeat([]byte("b"), keyLength)
+
+	block1, err := aes.NewCipher(key1)
 	if err != nil {
 		b.Fatal(err)
 	}
-	block2, err := aes.NewCipher(bytes.Repeat([]byte("b"), keyLength))
+	block2, err := aes.NewCipher(key2)
 	if err != nil {
 		b.Fatal(err)
 	}
 	p := value.NewPrefixTransformers(nil,
-		value.PrefixTransformer{Prefix: []byte("first:"), Transformer: newGCMTransformer(b, block1)},
-		value.PrefixTransformer{Prefix: []byte("second:"), Transformer: newGCMTransformer(b, block2)},
+		value.PrefixTransformer{Prefix: []byte("first:"), Transformer: f(b, block1, key1)},
+		value.PrefixTransformer{Prefix: []byte("second:"), Transformer: f(b, block2, key2)},
 	)
 
 	ctx := context.Background()
@@ -495,8 +569,8 @@ func benchmarkGCMRead(b *testing.B, keyLength int, valueLength int, expectStale
 	// reverse the key order if expecting stale
 	if expectStale {
 		p = value.NewPrefixTransformers(nil,
-			value.PrefixTransformer{Prefix: []byte("second:"), Transformer: newGCMTransformer(b, block2)},
-			value.PrefixTransformer{Prefix: []byte("first:"), Transformer: newGCMTransformer(b, block1)},
+			value.PrefixTransformer{Prefix: []byte("second:"), Transformer: f(b, block2, key2)},
+			value.PrefixTransformer{Prefix: []byte("first:"), Transformer: f(b, block1, key1)},
 		)
 	}
 
@@ -513,18 +587,21 @@ func benchmarkGCMRead(b *testing.B, keyLength int, valueLength int, expectStale
 	b.StopTimer()
 }
 
-func benchmarkGCMWrite(b *testing.B, keyLength int, valueLength int) {
-	block1, err := aes.NewCipher(bytes.Repeat([]byte("a"), keyLength))
+func benchmarkGCMWrite(b *testing.B, f transformerFunc, keyLength int, valueLength int) {
+	key1 := bytes.Repeat([]byte("a"), keyLength)
+	key2 := bytes.Repeat([]byte("b"), keyLength)
+
+	block1, err := aes.NewCipher(key1)
 	if err != nil {
 		b.Fatal(err)
 	}
-	block2, err := aes.NewCipher(bytes.Repeat([]byte("b"), keyLength))
+	block2, err := aes.NewCipher(key2)
 	if err != nil {
 		b.Fatal(err)
 	}
 	p := value.NewPrefixTransformers(nil,
-		value.PrefixTransformer{Prefix: []byte("first:"), Transformer: newGCMTransformer(b, block1)},
-		value.PrefixTransformer{Prefix: []byte("second:"), Transformer: newGCMTransformer(b, block2)},
+		value.PrefixTransformer{Prefix: []byte("first:"), Transformer: f(b, block1, key1)},
+		value.PrefixTransformer{Prefix: []byte("second:"), Transformer: f(b, block2, key2)},
 	)
 
 	ctx := context.Background()
@@ -657,31 +734,29 @@ func TestRoundTrip(t *testing.T) {
 	if err != nil {
 		t.Fatal(err)
 	}
-	aes32block, err := aes.NewCipher(bytes.Repeat([]byte("c"), 32))
+	key32 := bytes.Repeat([]byte("c"), 32)
+	aes32block, err := aes.NewCipher(key32)
 	if err != nil {
 		t.Fatal(err)
 	}
 
 	ctx := context.Background()
 	tests := []struct {
-		name    string
-		dataCtx value.Context
-		t       value.Transformer
+		name string
+		t    value.Transformer
 	}{
-		{name: "GCM 16 byte key", t: newGCMTransformer(t, aes16block)},
-		{name: "GCM 24 byte key", t: newGCMTransformer(t, aes24block)},
-		{name: "GCM 32 byte key", t: newGCMTransformer(t, aes32block)},
-		{name: "GCM 16 byte unsafe key", t: newGCMTransformerWithUniqueKeyUnsafeTest(t, aes16block)},
-		{name: "GCM 24 byte unsafe key", t: newGCMTransformerWithUniqueKeyUnsafeTest(t, aes24block)},
-		{name: "GCM 32 byte unsafe key", t: newGCMTransformerWithUniqueKeyUnsafeTest(t, aes32block)},
+		{name: "GCM 16 byte key", t: newGCMTransformer(t, aes16block, nil)},
+		{name: "GCM 24 byte key", t: newGCMTransformer(t, aes24block, nil)},
+		{name: "GCM 32 byte key", t: newGCMTransformer(t, aes32block, nil)},
+		{name: "GCM 16 byte unsafe key", t: newGCMTransformerWithUniqueKeyUnsafeTest(t, aes16block, nil)},
+		{name: "GCM 24 byte unsafe key", t: newGCMTransformerWithUniqueKeyUnsafeTest(t, aes24block, nil)},
+		{name: "GCM 32 byte unsafe key", t: newGCMTransformerWithUniqueKeyUnsafeTest(t, aes32block, nil)},
+		{name: "GCM 32 byte seed", t: newHKDFExtendedNonceGCMTransformerTest(t, nil, key32)},
 		{name: "CBC 32 byte key", t: NewCBCTransformer(aes32block)},
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			dataCtx := tt.dataCtx
-			if dataCtx == nil {
-				dataCtx = value.DefaultContext("")
-			}
+			dataCtx := value.DefaultContext("/foo/bar")
 			for _, l := range lengths {
 				data := make([]byte, l)
 				if _, err := io.ReadFull(rand.Reader, data); err != nil {
@@ -718,12 +793,14 @@ func TestRoundTrip(t *testing.T) {
 	}
 }
 
-type testingT interface {
-	Helper()
-	Fatal(...any)
+type namedTransformerFunc struct {
+	name string
+	f    transformerFunc
 }
 
-func newGCMTransformer(t testingT, block cipher.Block) value.Transformer {
+type transformerFunc func(t testing.TB, block cipher.Block, key []byte) value.Transformer
+
+func newGCMTransformer(t testing.TB, block cipher.Block, _ []byte) value.Transformer {
 	t.Helper()
 
 	transformer, err := NewGCMTransformer(block)
@@ -734,7 +811,7 @@ func newGCMTransformer(t testingT, block cipher.Block) value.Transformer {
 	return transformer
 }
 
-func newGCMTransformerWithUniqueKeyUnsafeTest(t testingT, block cipher.Block) value.Transformer {
+func newGCMTransformerWithUniqueKeyUnsafeTest(t testing.TB, block cipher.Block, _ []byte) value.Transformer {
 	t.Helper()
 
 	nonceGen := &nonceGenerator{fatal: die}
@@ -745,3 +822,14 @@ func newGCMTransformerWithUniqueKeyUnsafeTest(t testingT, block cipher.Block) va
 
 	return transformer
 }
+
+func newHKDFExtendedNonceGCMTransformerTest(t testing.TB, _ cipher.Block, key []byte) value.Transformer {
+	t.Helper()
+
+	transformer, err := NewHKDFExtendedNonceGCMTransformer(key)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	return transformer
+}
