@@ -30,8 +30,6 @@ import (
 	"github.com/google/go-cmp/cmp"
 	"github.com/spf13/cobra"
 
-	yaml "gopkg.in/yaml.v2"
-
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/cli-runtime/pkg/genericiooptions"
@@ -41,6 +39,7 @@ import (
 	"k8s.io/kubectl/pkg/cmd/create"
 	cmdtesting "k8s.io/kubectl/pkg/cmd/testing"
 	cmdutil "k8s.io/kubectl/pkg/cmd/util"
+	yaml "sigs.k8s.io/yaml/goyaml.v2"
 )
 
 type EditTestCase struct {
