@@ -76,10 +76,6 @@ const (
 	crdNamePlural                   = "testcrds"
 )
 
-var (
-	resourceConsumerImage = imageutils.GetE2EImage(imageutils.ResourceConsumer)
-)
-
 var (
 	// KindRC is the GVK for ReplicationController
 	KindRC = schema.GroupVersionKind{Version: "v1", Kind: "ReplicationController"}
@@ -144,7 +140,7 @@ func NewDynamicResourceConsumer(ctx context.Context, name, nsName string, kind s
 func getSidecarContainer(name string, cpuLimit, memLimit int64) v1.Container {
 	container := v1.Container{
 		Name:    name + "-sidecar",
-		Image:   resourceConsumerImage,
+		Image:   imageutils.GetE2EImage(imageutils.ResourceConsumer),
 		Command: []string{"/consumer", "-port=8081"},
 		Ports:   []v1.ContainerPort{{ContainerPort: 80}},
 	}
@@ -628,7 +624,7 @@ func runServiceAndWorkloadForResourceConsumer(ctx context.Context, c clientset.I
 
 	rcConfig := testutils.RCConfig{
 		Client:               c,
-		Image:                resourceConsumerImage,
+		Image:                imageutils.GetE2EImage(imageutils.ResourceConsumer),
 		Name:                 name,
 		Namespace:            ns,
 		Timeout:              timeoutRC,
