@@ -141,11 +141,10 @@ var _ = sigDescribe(feature.Windows, "GMSA Full", framework.WithSerial(), framew
 			// and authenticating with it.
 			ginkgo.By("checking that nltest /QUERY returns successfully")
 			var output string
-			gomega.Eventually(ctx, func() bool {
+			gomega.Eventually(ctx, func() error {
 				output, err = runKubectlExecInNamespace(f.Namespace.Name, podName, "--", "nltest", "/QUERY")
 				if err != nil {
-					framework.Logf("unable to run command in container via exec: %s", err)
-					return false
+					return fmt.Errorf("unable to run command in container via exec: %w", err)
 				}
 
 				if !isValidOutput(output) {
@@ -153,14 +152,12 @@ var _ = sigDescribe(feature.Windows, "GMSA Full", framework.WithSerial(), framew
 					// https://kubernetes.io/docs/tasks/configure-pod-container/configure-gmsa/#troubleshooting
 					output, err = runKubectlExecInNamespace(f.Namespace.Name, podName, "--", "nltest", fmt.Sprintf("/sc_reset:%s", gmsaDomain))
 					if err != nil {
-						framework.Logf("unable to run command in container via exec: %s", err)
-						return false
+						return fmt.Errorf("unable to run command in container via exec: %w", err)
 					}
-					framework.Logf("failed to connect to domain; tried resetting the domain, output:\n%s", string(output))
-					return false
+					return fmt.Errorf("failed to connect to domain; tried resetting the domain, output:\n%v", string(output))
 				}
-				return true
-			}, 1*time.Minute, 1*time.Second).Should(gomega.BeTrue())
+				return nil
+			}, 1*time.Minute, 1*time.Second).Should(gomega.Succeed())
 		})
 
 		ginkgo.It("can read and write file to remote SMB folder", func(ctx context.Context) {
@@ -208,16 +205,16 @@ var _ = sigDescribe(feature.Windows, "GMSA Full", framework.WithSerial(), framew
 
 			ginkgo.By("checking that file can be read and write from the remote folder successfully")
 			filePath := fmt.Sprintf("\\\\%s\\%s\\write-test-%s.txt", gmsaDomainIP, gmsaSharedFolder, string(uuid.NewUUID())[0:4])
-			gomega.Eventually(ctx, func() bool {
+
+			gomega.Eventually(ctx, func() error {
 				// The filePath is a remote folder, do not change the format of it
 				_, _ = runKubectlExecInNamespace(f.Namespace.Name, podName, "--", "powershell.exe", "-Command", "echo 'This is a test file.' > "+filePath)
-				output, err := runKubectlExecInNamespace(f.Namespace.Name, podName, "powershell.exe", "--", "cat", filePath)
+				_, err := runKubectlExecInNamespace(f.Namespace.Name, podName, "powershell.exe", "--", "cat", filePath)
 				if err != nil {
-					framework.Logf("unable to get file from AD server: %s", err)
-					return false
+					return err
 				}
-				return strings.Contains(output, "This is a test file.")
-			}, 1*time.Minute, 1*time.Second).Should(gomega.BeTrue())
+				return nil
+			}, 1*time.Minute, 1*time.Second).Should(gomega.Succeed())
 
 		})
 	})
