@@ -77,18 +77,24 @@ var _ = SIGDescribe("Density", framework.WithSerial(), framework.WithSlow(), fun
 	})
 
 	f.Context("create a batch of pods", framework.WithFlaky(), func() {
-		// TODO(coufon): the values are generous, set more precise limits with benchmark data
-		// and add more tests
+		// If this test case fails with am error similar to
+		// "container "runtime": expected 95th% usage < 0.900; got 0.941",
+		// it is likely that cpuLimits or memLimits need to be increased.
+		// Actual resource usage values can be found in the test output, e.g.:
+		// I1029 11:28:15.671913 1005 resource_usage_test.go:206] CPU usage of containers:
+		// container 50th% 90th% 95th% 99th% 100th%
+		// "runtime" 0.004 0.941 0.941 0.941 0.976
+		// "kubelet" 0.009 0.082 0.082 0.082 0.101
 		dTests := []densityTest{
 			{
 				podsNr:   10,
 				interval: 0 * time.Millisecond,
 				cpuLimits: e2ekubelet.ContainersCPUSummary{
-					kubeletstatsv1alpha1.SystemContainerKubelet: {0.50: 0.30, 0.95: 0.50},
-					kubeletstatsv1alpha1.SystemContainerRuntime: {0.50: 0.40, 0.95: 0.60},
+					kubeletstatsv1alpha1.SystemContainerKubelet: {0.50: 0.1, 0.95: 0.20},
+					kubeletstatsv1alpha1.SystemContainerRuntime: {0.50: 0.1, 0.95: 1.5},
 				},
 				memLimits: e2ekubelet.ResourceUsagePerContainer{
-					kubeletstatsv1alpha1.SystemContainerKubelet: &e2ekubelet.ContainerResourceUsage{MemoryRSSInBytes: 100 * 1024 * 1024},
+					kubeletstatsv1alpha1.SystemContainerKubelet: &e2ekubelet.ContainerResourceUsage{MemoryRSSInBytes: 50 * 1024 * 1024},
 					kubeletstatsv1alpha1.SystemContainerRuntime: &e2ekubelet.ContainerResourceUsage{MemoryRSSInBytes: 500 * 1024 * 1024},
 				},
 				// percentile limit of single pod startup latency
@@ -366,7 +372,7 @@ func runDensityBatchTest(ctx context.Context, f *framework.Framework, rc *Resour
 
 	gomega.Eventually(ctx, func() bool {
 		return len(watchTimes) == testArg.podsNr
-	}, 10*time.Minute, 10*time.Second).Should(gomega.BeTrue())
+	}, 10*time.Minute, 10*time.Second).Should(gomega.BeTrueBecause("All pods should be observed by the watch."))
 
 	if len(watchTimes) < testArg.podsNr {
 		framework.Failf("Timeout reached waiting for all Pods to be observed by the watch.")
