@@ -20,7 +20,6 @@ import (
 	"encoding/json"
 	"errors"
 	"io"
-	"io/ioutil"
 	"net/http"
 	"os"
 	"time"
@@ -46,7 +45,7 @@ func getCredentials(tokenEndpoint string, r io.Reader, w io.Writer) error {
 		tokenEndpoint: tokenEndpoint,
 	}
 
-	data, err := ioutil.ReadAll(r)
+	data, err := io.ReadAll(r)
 	if err != nil {
 		return err
 	}
